function MuscleModelOptimization3D_reconstructedActivations_v4
% Use this version
clear all
close all

%% Settings
MakePredictions = 0;
bothSpeeds = 1;
useMedianTrial = 0;
LoadOldResults = 1; % load previous results and use as initial guess
LoadFileName = 'params_Subject1_hipWeighted_v4pt3_bothSpeeds.mat'; % load old model parameter values
LoadSynergiesName = 'synergies_Subject1_hipWeighted_v4pt3_bothSpeeds.mat'; % load previous synergy weights and commands
RunOpt = 0; % run optimization, 0 will show results for initial guess/loaded results
optType = 1; % which optimizer is being used 1 for fmincon, 2 for pso, 3 for lsqnonlin. Only fmincon is set up right now
GeometricAdjustments = 1;
StepWiseOptimizations = 1; % split optimization into smaller optimizations for better convergence
SaveData = 0; % save data !!!Will overwrite old file if name is not changed!!!
SaveNameParams = 'params_Subject1_hipWeighted_v4pt3_bothSpeeds.mat'; % only model parameters get saved here
SaveNameResults = 'results_Subject1_hipWeighted_v4pt3_bothSpeeds.mat'; % all results for the simulation get saved here
SaveNameSynergies = 'synergies_Subject1_hipWeighted_v4pt3_bothSpeeds.mat'; % synergy weights and commands get saved here
makeParameterChangePlotsSwitch = 0; % activates design variable value plots
makeIndividualMuscMomentPlotsSwitch = 0; % activates individual muscle moment plots
makeMomentMatchingPlotsSwitch = 1; % activates moment matching plots
makeMuscleParameterPlotsSwitch = 0; % activates lmt, vmt, moment arm plots
makeExcitationsPlotsSwitch = 1; % activates excitation plots
makeSynergyCommandPlotsSwitch = 0;
makePassiveMomentMatchingPlotsSwitch = 1;
makeJointAnglePlotsSwitch = 0;
makeEmgPlotsSwitch = 0;
hpf = 40; % high pass filter cutoff for the EMG data
lpfMax = 3.5;
numIter = 50; % iteration limit for the stepwise optimizations
numIterLarge = 150; %200; % iteration limit for the final overall optimization
algorithmfmincon = 'sqp'; % optimizer type for fmincon
TrialsStep = 1; % Reduces the number of trials used, Might not work any more...
nJoints = 5; % number of joints in model
Leg = 'both'; % which leg is being optimized, right, left, or both
SampleStep = 1; % resamples data to ever SampleStep points using 1:SampleStep:end. Must be 1, 2, 5 or 10.
nptsLong = (141-1)/SampleStep+1; % number of sample points in gait cycle with padded beginnings and ends
nptsShort = (101-1)/SampleStep+1; % number of sample points in gait cycle without pading

% Specify number of muscles and EMG signals for each muscle
nEmg = 14; % Number of EMG signals recorded
nMuscEmg = 31; % Number of muscles with EMG available
nMuscReconst = 5; % Number of muscles which will have activations reconstructed using synergies
nReconstDup = 1; % Number of reconstructed muscle activations which are duplicates of other reconstructed activations
nMuscTotal = nMuscEmg + nMuscReconst; % Total number of muscles in the model
haveEmg = 1:31; % Which mucles (out of total) have EMG available
nSyn = 5; % Number of synergies to use

% Subject height and mass for max muscle force calculation
height = 1.74; % m
mass = 78.86; % kg

% Load processed trial data, includes raw EMG, joint angles, joint
% velocities, etc
load Subject1_v4pt3_etmData_bothLegs_replacedRMGas_outliersRemoved.mat
% Load initial guesses for the muscle fits
load Subject1_v4pt1_MuscleFits_lhs.mat
% Load passive muscle forces and associated joint angles from Silder et
% al. 2007 (with Thelen as corresponding author)
load DigitizedPassiveMoments.mat
JAnglesThelen = JAngles;
PassiveMThelen = PassiveM;


% Compile matrices of trials to be used
% IMPORTANT: MUST HAVE LEFT FOOT TRIALS BEFORE RIGHT FOOT for consistency
% with plots and saving of data
if useMedianTrial
    load medianTrials.mat
    Data = [etmData_l(medianTrial_l),etmData_r(medianTrial_r)];
    nTrialsToUse = 1;
else
    numTrialsPerLeg = 40; %30
    if MakePredictions
        Data = [etmData_l(1:numTrialsPerLeg),...
            etmData_r(1:numTrialsPerLeg)];
    else
        if bothSpeeds
            Data = [etmData_l([25:(24+numTrialsPerLeg/2),75:(74+numTrialsPerLeg/2)]),...
                etmData_r([25:(24+numTrialsPerLeg/2),75:(74+numTrialsPerLeg/2)])]; 
        else
            Data = [etmData_l(75:(74+numTrialsPerLeg)),...
                etmData_r(75:(74+numTrialsPerLeg))];
%             Data = [etmData_l(20:(19+numTrialsPerLeg)),...
%                 etmData_r(20:(19+numTrialsPerLeg))]; 
        end
        
    end
    % Determine how many trials to use
    nTrialsToUse = numTrialsPerLeg;
end

% Specify how many trials are to be used from left or right foot
nTrials_l = nTrialsToUse;
nTrials_r = nTrialsToUse;

% Determine number of trials used in optimization
nTrialsOrig = nTrials_l + nTrials_r;
% Determine number of trials used after trials are downsampled
nTrials = ceil(nTrialsOrig/TrialsStep);
% print number of trials being used to command line
fprintf(['\nUsing ' num2str(nTrials) ' trials for model calibration.\n']);

% Append all individual muscle coefficients into a single dimension array form
coefsFit = MuscleFits.Coefs;
coefsOrig = [];
for iii = 1:nMuscTotal
    coefsOrig = [coefsOrig; coefsFit{iii}];
end

% Number of coefficients used in the muscle geometry fits
ncoefsFit = length(coefsOrig);

% determine number of time frames
nFrames = nptsShort*nTrials;

% Save new variables
optParams.nTrials = nTrials;
optParams.nMuscTotal = nMuscTotal;
optParams.nMuscEmg = nMuscEmg;
optParams.nMuscReconst = nMuscReconst;
optParams.nReconstDup = nReconstDup;
optParams.haveEmg = haveEmg;
optParams.nFrames = nFrames;
optParams.nSyn = nSyn;
optParams.nTrials_l = nTrials_l;
optParams.nTrials_r = nTrials_r;
optParams.nTrialsToUse = nTrialsToUse;


%% Load data
% Preallocate memory
Time = zeros(nptsLong,nTrials); % Time vector for each trial
JAngles = zeros(nptsShort,6,nTrials); % joint angles for each trial in radians
JVels = zeros(nptsShort,6,nTrials); % joint velocities for each trial in radians/s
HipFEMA = zeros(nptsShort,nMuscTotal,nTrials); % Hip FE moment arms in meters
HipAAMA = zeros(nptsShort,nMuscTotal,nTrials); % Hip AA moment arms in meters
KneeMA = zeros(nptsShort,nMuscTotal,nTrials); % Knee FE moment arms in meters
AnkleMA = zeros(nptsShort,nMuscTotal,nTrials); % Ankle moment arms in meters
SubtalarMA = zeros(nptsShort,nMuscTotal,nTrials); % Subtalar moment arms in meters
Lmt = zeros(nptsShort,nMuscTotal,nTrials); % Muscle tendon lengths in meters
Vmt = zeros(nptsShort,nMuscTotal,nTrials); % Muscle tendon velocities in meters/s
IDloads = zeros(nptsShort,5,nTrials); % Inverse dynamics loads in N-m

% Store data in new format (Time Frames)x(Muscle or DOF)x(Trial)
for iii = 1:TrialsStep:nTrialsOrig
    jjj = ceil(iii/TrialsStep);
    
    Time(:,jjj) = Data(iii).Time(1:SampleStep:141);
    JAngles(:,:,jjj) = Data(iii).JointAngles(21:SampleStep:121,1:6)*pi/180;
    JVels(:,:,jjj) = Data(iii).JointVelocities(21:SampleStep:121,1:6)*pi/180;
    HipFEMA(:,:,jjj) = Data(iii).HipFEMomentArms(21:SampleStep:121,:);
    HipAAMA(:,:,jjj) = Data(iii).HipAAMomentArms(21:SampleStep:121,:);
    KneeMA(:,:,jjj) = Data(iii).KneeMomentArms(21:SampleStep:121,:);
    AnkleMA(:,:,jjj) = Data(iii).AnkleMomentArms(21:SampleStep:121,:);
    SubtalarMA(:,:,jjj) = Data(iii).SubtalarMomentArms(21:SampleStep:121,:);
    Lmt(:,:,jjj) = Data(iii).MuscleTendonLengths(21:SampleStep:121,:);
    Vmt(:,:,jjj) = Data(iii).MuscleTendonVelocities(21:SampleStep:121,:);
    IDloads(:,:,jjj) = Data(iii).InverseDynamicsLoads(21:SampleStep:121,:);
end

% save original curves for plotting purposes
Lmtorig = Lmt;
HipMAorig = HipFEMA;
HipAAMAorig = HipAAMA;
KneeMAorig = KneeMA;
AnkleMAorig = AnkleMA;
SubtalarMAorig = SubtalarMA;


%% Set muscle similarity parameters
% Set similarity pairs that keep parameters for similar muscles close
% Each row is a pair of muscles
% NOTE: these pairing depend on muscle order
optParams.ActivationPairs = ... %inclues time delay, activation/deactivation, and EMG scale factor
    {
    1:6 % Adductors
    7:9 % Gluteus maximumus
    10:12 % Gluteus medius
    13:15 % Gluteus minimus
    17:18 % Semimem/semiten
    19:20 % Biceps Fem lh/sh
    21:24 % Quads
    25:26 % Gastrocs
    28:30 % Peroneals
    }; %NOTE: These have had the muscle indices without EMG removed

optParams.lmtildaPairs = ... % penalizes differences between lmtilda curves
    {
    7:9 % Gluteus maximumus
    10:12 % Gluteus medius
    13:15 % Gluteus minimus
    22:24 % Quads (not rec fem)
    25:26 % Gastrocs
    };

optParams.HipMAPairs = cell(0,2); % penalizes differences between Hip FE MA curves
optParams.HipAAMAPairs = cell(0,2); % penalizes differences between Hip AA MA curves
optParams.KneeMAPairs = { % penalizes differences between Knee MA curves
    17:18 % Semimem/semiten
    19:20 % Biceps Fem lh/sh
    21:24}; % Quads
optParams.AnkleMAPairs = {[25:26 31] % penalizes differences between Ankle MA curves
    28:29}; % peroneals except tertius which has different moment arm
optParams.SubtalarMAPairs = {[25:26 31] % penalizes differences between Subtalar MA curves
    28:30}; % peroneals including tertius


%% Process the EMG data if needed
% NOTE (Nathan): For this particular data set, make sure that you have
% replaced the R Gas Med EMG data with the R Soleus data (as R GasMed was
% bad)
% Preallocate memory
emg = zeros(nptsLong,nEmg,nTrials);
lpf = zeros(1,nTrials);
TimeEMG = Time;

for iii = 1:TrialsStep:nTrialsOrig
    jjj = ceil(iii/TrialsStep);
    
    % determine time length from heel strike to heel strike (not including
    % padding on beginning and end of cycle)
    tf = (TimeEMG(end,jjj)-TimeEMG(1,jjj))*(nptsShort/nptsLong);
    lpf(iii) = lpfMax/tf;
    
    % catch a common step parsing error in preprocessing
    if tf > 10
        keyboard
    end
    
    % process the EMG data
    EMGtrial = processEMG(Data(iii).EMG, 1/1000, hpf, lpf(iii));
    
    % find original final time of trial
    tf = (length(EMGtrial)-1)/1000;
    
    % create normalized emg signals time vectors
    TimeEMGtrial = 0:(1/1000):tf;
    TimeEMG(:,iii) = linspace(0,tf,nptsLong);
    % Normalize to nptsLong time frames
    emg(:,:,jjj) = spline(TimeEMGtrial, EMGtrial', TimeEMG(:,iii))';
end

% Plot EMG for visualization purposes
% EMG = permute(EMG, [1 3 2]);
% plot(EMG(:,1:300,4))
%
% keyboard

% Reorder EMG and duplicate signals for similar muscles
emgAll = emg(:,[6*ones(1,6) 7*ones(1,3) 9*ones(1,6) 8 10*ones(1,2) 5*ones(1,2) 4 11 3*ones(1,2) 2 12 1 13*ones(1,3) 14],:);

% % Duplicate EMG signals for similar muscles
% EMG = EMG(:,[ones(1,6) 2*ones(1,3) 3*ones(1,6) 4*ones(1,2) 5*ones(1,2) 6*ones(1,2) 7:9 9 10*ones(1,2) 11:12 13*ones(1,3) 14:16],:);


%% Normalize EMG

% split EMG into left and right side
EMG_l = emgAll(:,:,1:nTrials_l);
EMG_r = emgAll(:,:,(nTrials_l+1):end);

% Remove EMG offset for each trial
EMGmins_l = min(EMG_l(21:121,:,:),[],1);
EMG_l = EMG_l-EMGmins_l(ones(141,1),:,:);
EMGmins_r = min(EMG_r(21:121,:,:),[],1);
EMG_r = EMG_r-EMGmins_r(ones(141,1),:,:);

% activate to determine new scale factors for EMG data
if 1
    EMGmin_r = zeros(1,nMuscEmg);
    EMGmax_r = zeros(1,nMuscEmg);
    EMGmin_l = zeros(1,nMuscEmg);
    EMGmax_l = zeros(1,nMuscEmg);
    
    for iii = 1:nMuscEmg
        EMGmin_r(iii) = min(min(EMG_r(11:121,iii,:)));
        EMGmax_r(iii) = max(max(EMG_r(11:121,iii,:)-EMGmin_r(iii)));
        EMG_r(:,iii,:) = (EMG_r(:,iii,:)-EMGmin_r(iii))/EMGmax_r(iii);
        EMGmin_l(iii) = min(min(EMG_l(11:121,iii,:)));
        EMGmax_l(iii) = max(max(EMG_l(11:121,iii,:)-EMGmin_l(iii)));
        EMG_l(:,iii,:) = (EMG_l(:,iii,:)-EMGmin_l(iii))/EMGmax_l(iii);
    end
    
    save EMGmaxima_r.mat EMGmin_r EMGmax_r
    save EMGmaxima_l.mat EMGmin_l EMGmax_l
    
else
    % Load EMG scale factors determined for all trials
    load EMGmaxima_l.mat
    load EMGmaxima_r.mat
    
    % Normalize EMG data
    for iii = 1:nMuscEmg
        EMG_l(:,iii,:) = (EMG_l(:,iii,:)-EMGmin_l(iii))/EMGmax_l(iii);
        EMG_r(:,iii,:) = (EMG_r(:,iii,:)-EMGmin_r(iii))/EMGmax_r(iii);
    end
end
emgAll = cat(3,EMG_l,EMG_r);

% determine median trials at each gait speed
if useMedianTrial
    % Remove duplicate muscle signals
    EMG_l = EMG_l(:,[1 7 10 16 18 20 21 22 24 26 27 30],:); %TO DO: These are not correct numbers
    EMG_r = EMG_r(:,[1 7 10 16 18 20 21 22 24 26 27 30],:);

    % Determine median curves for each muscle
    EMGmed_l = median(EMG_l(:,:,1:nTrials_l),3);
    EMGmed_r = median(EMG_r(:,:,1:nTrials_r),3);

    % Find sum of squares of differences between the median curve and each
    % trial for all muscles
    EMGdev_l = sum(sum((EMG_l(:,:,1:nTrials_l)-EMGmed_l(:,:,ones(nTrials_l,1))).^2));
    EMGdev_r = sum(sum((EMG_r(:,:,1:nTrials_r)-EMGmed_r(:,:,ones(nTrials_r,1))).^2));

    % Select trial with minimum sum of squares of differences
    [~,medianTrial_l] = min(EMGdev_l,[],3);
    [~,medianTrial_r] = min(EMGdev_r,[],3);

    save medianTrials.mat  medianTrial_l medianTrial_r

end

% remove EMG values less than 0
emgAll(emgAll<0) = 0;


%% Create EMG splines
EMGsplines = cell(1,nTrials);

for jjj = 1:nTrials
    EMGsplines{jjj} = spline(Time(1:4:end,jjj),emgAll(1:4:end,:,jjj)'); % Use every other frame to improve speed when evaluating spline
end

optParams.EMG = permute(emgAll, [1 3 2]);
optParams.EMGsplines = EMGsplines;


%% Set initial muscle parameter values

% Muscle parameters from Arnold et al. 2010
lmo =   [10.3 10.8 17.7 15.6 13.8 10.6 14.7 15.7 16.7 7.3 7.3 7.3 6.8 5.6 3.8 9.5 6.9 19.3 9.8 11 7.6 9.7 9.9 9.9 5.9 5.1 6.8 4.5 5.1 7.9 4.4 10.7 11.7 3.8 6.9 4.5]/100;
lts =   [3.6 13.0 9.0 22.1 4.8 4.3 5.0 7.3 7.0 5.7 6.6 4.6 1.6 2.6 5.1 45.0 37.8 24.5 32.2 10.4 34.6 11.2 13 10.6 38.2 40.1 24.1 14.9 33.3 10.0 28.2 9.4 9.7 28.2 36.7 37.8]/100;
alpha = [6.1 7.1 13.8 11.9 14.7 22.2 21.1 21.9 22.8 20.5 20.5 20.5 10.0 0 1.0 3.0 15.1 12.9 11.6 12.3 13.9 29.6 18.4 4.5 12.0 9.9 9.6 11.5 14.1 13.0 28.3 14.3 10.7 13.7 10.8 13.6]*pi/180;
% Fmax = 2*[303.7 399.5 324.2 324.2 324.2 324.2 546.1 780.5 526.1 881.1 616.5 702.0 180.0 190.0 215.0 621.9 479.7 1162.7 301.9 705.2 315.8 848.8 1443.7 2255.4 1024.2 606.4 1308.0 673.7 905.6 305.9 653.3 90.0 3585.9 345.4 274.4];

sigma = 610e3;
LegVol = (47*height*mass+1285)/100^3; % NOTE (Nathan): Subject height and mass will need to be changed for each subject
VolFraction = [1.47 2.26...
    1.97 1.97 1.97 1.97 ...
    3.52 5.02 3.39 ...
    1.82 1.27 1.45...
    0.45 0.48 0.54...
    0.89 3.46 2.60 2.92 1.40 3.79 6.06 11.66 3.84 2.11 3.62 1.91...
    0.53 1.14 0.16 6.21...
    2.48 3.80 1.49 0.97 0.43];
VolFraction = VolFraction/sum(VolFraction);
Vmusc = LegVol*VolFraction;


%% Construct matrices for calculating moment arms and joint velocities
% define muscle reference matrix that defines which joint(s) a muscle
% actuates
MuscRef = zeros(1,nMuscTotal);
MuscRef([1:16,32:33]) = 1; % Hip FE and AA
MuscRef([17:19 21]) = 2; % Hip FE and AA and Knee FE
MuscRef([20 22:24]) = 3; % Knee FE
MuscRef([25:26]) = 4; % Knee FE, Ankle and Subtalar
MuscRef([27:31,34:36]) = 5; % Ankle and Subtalar


JAnglesReshaped = reshape(permute(JAngles, [1 3 2]), nptsShort*nTrials, 6);
JVelsReshaped = reshape(permute(JVels, [1 3 2]), nptsShort*nTrials, 6);

optParams.Mat = buildMuscleMatrices(JAnglesReshaped,JVelsReshaped);
optParams.MuscRef = MuscRef;
optParams.JAnglesReshaped = JAnglesReshaped;
optParams.jointAngles = JAngles;


%% Setup matrices for calculating passive forces from thelen data

% Remove high knee flexion angle from hip moment matching
PassiveMThelen{1} = reshape(PassiveMThelen{1},numel(PassiveMThelen{1})/4,4);
PassiveMThelen{1}(:,4) = [];
PassiveMThelen{1} = PassiveMThelen{1}(:);

% Remove high knee flexion from knee moment matching
PassiveMThelen{2} = reshape(PassiveMThelen{2},numel(PassiveMThelen{2})/4,4);
PassiveMThelen{2}(92:121,:) = [];
PassiveMThelen{2} = PassiveMThelen{2}(:);

% Combine the passive moments
optParams.PassiveMThelen = [PassiveMThelen{1} zeros(length(PassiveMThelen{1}),4);...
    zeros(length(PassiveMThelen{2}),2) PassiveMThelen{2} zeros(length(PassiveMThelen{2}),2);...
    zeros(length(PassiveMThelen{3}),3) PassiveMThelen{3} zeros(length(PassiveMThelen{3}),1)];

% Remove high knee flexion angle from hip moment matching
JAnglesThelen{1} = reshape(JAnglesThelen{1},numel(JAnglesThelen{1})/(4*6),4,6);
JAnglesThelen{1}(:,4,:) = [];
JAnglesThelen{1} = reshape(JAnglesThelen{1},numel(JAnglesThelen{1})/(6),6);

% Remove high knee flexion from knee moment matching
JAnglesThelen{2} = reshape(JAnglesThelen{2},numel(JAnglesThelen{2})/(4*6),4,6);
JAnglesThelen{2}(92:121,:,:) = [];
JAnglesThelen{2} = reshape(JAnglesThelen{2},numel(JAnglesThelen{2})/(6),6);

%Combine the joint angles
JAnglesThelen = [JAnglesThelen{1}; JAnglesThelen{2}; JAnglesThelen{3}]*pi/180;

% Add offset to the angles to account for differences in models
JAnglesThelen(:,1) = JAnglesThelen(:,1)-0*pi/180;
JAnglesThelen(:,2) = JAnglesThelen(:,2)-0*pi/180;
JAnglesThelen(:,4) = JAnglesThelen(:,4)+0*pi/180;

JVelsThelen = zeros(size(JAnglesThelen));

nFramesThelen = length(JAnglesThelen);
nFramesAllThelen = nFramesThelen;

optParams.aThelen = zeros(nFramesAllThelen,nMuscTotal);
optParams.onesColThelen = ones(nFramesAllThelen,1);
optParams.nframesAllThelen = nFramesAllThelen;

optParams.MatThelen = buildMuscleMatrices(JAnglesThelen, JVelsThelen);


%% Assign values to optimization parameters matrix

% Assign values to optimization structure
optParams.Time = Time;
optParams.JAngles = reshape(permute(JAngles, [1 3 2]), nptsShort*nTrials, 6);
optParams.JVels = reshape(permute(JVels, [1 3 2]), nptsShort*nTrials, 6);
optParams.HipMA = reshape(permute(HipFEMA, [1 3 2]), nptsShort*nTrials, nMuscTotal);
optParams.HipAAMA = reshape(permute(HipAAMA, [1 3 2]), nptsShort*nTrials, nMuscTotal);
optParams.KneeMA = reshape(permute(KneeMA, [1 3 2]), nptsShort*nTrials, nMuscTotal);
optParams.AnkleMA = reshape(permute(AnkleMA, [1 3 2]), nptsShort*nTrials, nMuscTotal);
optParams.SubtalarMA = reshape(permute(SubtalarMA, [1 3 2]), nptsShort*nTrials, nMuscTotal);
optParams.Lmt = reshape(permute(Lmt, [1 3 2]), nptsShort*nTrials, nMuscTotal);
optParams.Vmt = reshape(permute(Vmt, [1 3 2]), nptsShort*nTrials, nMuscTotal);
optParams.IDloads = reshape(permute(IDloads, [1 3 2]), nptsShort*nTrials, 5);
% optParams.ACTsplines = ACTsplines;
optParams.lmo = lmo;
optParams.lts = lts;
optParams.alpha = alpha;
% optParams.Fmax = Fmax;
optParams.Vmusc = Vmusc;
optParams.sigma = sigma;
optParams.nframesAll = nptsShort*nTrials;
optParams.onesCol = ones(nptsShort*nTrials,1);
optParams.SampleStep = SampleStep;
optParams.nptsShort = nptsShort;
optParams.nptsLong = nptsLong;
optParams.coefsOrig = coefsOrig;
optParams.ncoefsFit = ncoefsFit;
optParams.nTrials_l = nTrials_l;
optParams.nTrials_r = nTrials_r;
optParams.optType = optType;
optParams.MuscLabels = Data(1).OpensimMuscleLabels;
optParams.muscLabelsEmg = optParams.MuscLabels(haveEmg);
optParams.GeometricAdjustments = GeometricAdjustments;

% load muscleGeoCoefScaleFactors.mat
optParams.coefsScaleFactors = 0.1; %coefsScaleFactors/10;

% load precalibrated lmo and lts values
load Subject1_v4pt3_scaledLmoLts.mat
optParams.lmo = lmo;
optParams.lts = lts;


%% Construct Optimization Guess and Bounds
if LoadOldResults
    load(LoadFileName)
    
    guess.reconstSynWeights = params(1,1:2*nSyn*(nMuscReconst-nReconstDup));
    index = 2*nSyn*(nMuscReconst-nReconstDup);
    
    guess.tDelay = params(1,(index+1):(index+2));
    index = index+2;

    guess.tAct = params(1,(index+1):(index+nMuscEmg));
    index = index + nMuscEmg;

    guess.aNonlin = params(1,(index+1):(index+nMuscEmg));
    index = index + nMuscEmg;
    
    guess.emgScale = params(1,(index+1):(index + 2*nMuscEmg));
    index = index + 2*nMuscEmg;
    
    guess.lmoScale = params(1,(index+1):(index + nMuscTotal));
    index = index + nMuscTotal;
    
    guess.ltsScale = params(1,(index+1):(index + nMuscTotal));
    index = index + nMuscTotal;
    
    if GeometricAdjustments
        if length(params) == (index + ncoefsFit)
            guess.coefsOffset = params(1,(index+1):(index + ncoefsFit));
        else
            guess.coefsOffset = zeros(1,ncoefsFit);
        end
    end
    
    load(LoadSynergiesName)
else
    guess.reconstSynWeights = 0.25*ones(1,2*nSyn*(nMuscReconst-nReconstDup));
    guess.tDelay = 0.5*ones(1,2);
    guess.tAct = 1.5*ones(1,1*nMuscEmg);
    guess.aNonlin = zeros(1,1*nMuscEmg);
    guess.emgScale = .25*ones(1,2*nMuscEmg);
    guess.lmoScale = ones(1,nMuscTotal);
    guess.ltsScale = ones(1,nMuscTotal);
    if GeometricAdjustments
        guess.coefsOffset = zeros(1,ncoefsFit);
    end
    
    % Create initial synergies guess
    optParams.tDelay = guess.tDelay/10;
    optParams.tAct = guess.tAct/100;
    optParams.tDeact = 4*optParams.tAct;
    optParams.aNonlin = guess.aNonlin;
    optParams.emgScale = guess.emgScale;

    [a,~] = EMGtoAct(optParams);
    a_l = a(1:(nTrials_l*nptsShort),:);
    a_r = a((nTrials_l*nptsShort + 1):end,:);
    a_l_unitVar = zeros(size(a_l));
    a_r_unitVar = zeros(size(a_r));
    % Divide each signal by std to get unitized variablity?
    for iii = 1:nMuscEmg
        a_l_unitVar(:,iii) = a_l(:,iii)/std(a_l(:,iii));
        a_r_unitVar(:,iii) = a_r(:,iii)/std(a_r(:,iii));
    end
    [synergyCommands_l,synergyWeights_l,rms_l] = nnmf(a_l_unitVar,nSyn);
    [synergyCommands_r,synergyWeights_r,rms_r] = nnmf(a_r_unitVar,nSyn);
    
    rms_l_best = rms_l;
    rms_r_best = rms_r;
    synergyCommands_l_best = synergyCommands_l;
    synergyCommands_r_best = synergyCommands_r;
    synergyWeights_l_best = synergyWeights_l;
    synergyWeights_r_best = synergyWeights_r;
    for iii = 1:10
        [synergyCommands_l,synergyWeights_l,rms_l] = nnmf(a_l_unitVar,nSyn);
        [synergyCommands_r,synergyWeights_r,rms_r] = nnmf(a_r_unitVar,nSyn);
        
        if rms_l < rms_l_best
            rms_l_best = rms_l;
            synergyCommands_l_best = synergyCommands_l;
            synergyWeights_l_best = synergyWeights_l;
        end
        if rms_r < rms_r_best
            rms_r_best = rms_r;
            synergyCommands_r_best = synergyCommands_r;
            synergyWeights_r_best = synergyWeights_r;
        end
    end
    
    synergyCommands_l = synergyCommands_l_best;
    synergyCommands_r = synergyCommands_r_best;
    synergyWeights_l = synergyWeights_l_best;
    synergyWeights_r = synergyWeights_r_best;
    
    synergyCommandsNorm_l = zeros(size(synergyCommands_l));
    synergyWeightsNorm_l = zeros(size(synergyWeights_l));
    synergyCommandsNorm_r = zeros(size(synergyCommands_r));
    synergyWeightsNorm_r = zeros(size(synergyWeights_r));
    for iii = 1:nSyn
        lMax = max(synergyCommands_l(:,iii));
        rMax = max(synergyCommands_r(:,iii));
        synergyCommandsNorm_l(:,iii) = synergyCommands_l(:,iii)/lMax;
        synergyWeightsNorm_l(iii,:) = synergyWeights_l(iii,:)*lMax;
        synergyCommandsNorm_r(:,iii) = synergyCommands_r(:,iii)/rMax;
        synergyWeightsNorm_r(iii,:) = synergyWeights_r(iii,:)*rMax;
    end
    
end

upBounds.reconstSynWeights = 1*ones(1,2*nSyn*(nMuscReconst-nReconstDup)); % Was at 1
lowBounds.reconstSynWeights = zeros(1,2*nSyn*(nMuscReconst-nReconstDup));
upBounds.tDelay = 1*ones(1,2); % This was at 1.25
lowBounds.tDelay = 0*ones(1,2);
upBounds.tAct = 3.5*ones(1,1*nMuscEmg);
lowBounds.tAct = 0.75*ones(1,1*nMuscEmg);
upBounds.aNonlin = 0.35*ones(1,1*nMuscEmg);
lowBounds.aNonlin = zeros(1,1*nMuscEmg);
upBounds.emgScale = 1*ones(1,2*nMuscEmg);
lowBounds.emgScale = .05*ones(1,2*nMuscEmg);
upBounds.lmoScale = 1.25*ones(1,nMuscTotal);
lowBounds.lmoScale = .75*ones(1,nMuscTotal);
upBounds.ltsScale = 1.25*ones(1,nMuscTotal);
lowBounds.ltsScale = .75*ones(1,nMuscTotal);
if GeometricAdjustments
    upBounds.coefsOffset = 100*ones(1,ncoefsFit); %1000
    lowBounds.coefsOffset = -100*ones(1,ncoefsFit); %-1000
end


% setup optimization options
options = optimset('display','iter','MaxIter',numIter,'MaxFunEvals',100000,'Algorithm',algorithmfmincon,'UseParallel','always','Hessian','lbfgs');
optionsLarge = optimset('display','iter','MaxIter',numIterLarge,'MaxFunEvals',1000000,'Algorithm',algorithmfmincon,'UseParallel','always','Hessian','lbfgs');


%% Run overall model optimization
% if runopt is set to zero, will output results from guess

if RunOpt
    % Splits up optimization into three phases
    % 1: optimize EMG to activation parameters and reconstruct activations for muscles missing EMG
    % 2: optimize lmo, lts
    % 3: optimize the muscle geometric coefficients
    if StepWiseOptimizations
        for jjj = 1:2
            % Step 1
            % Generate initial guess for the synergy commands and weights
            optParams.tDelay = guess.tDelay/10;
            optParams.tAct = guess.tAct/100;
            optParams.tDeact = 4*optParams.tAct;
            optParams.aNonlin = guess.aNonlin;
            optParams.emgScale = guess.emgScale;
            
            [a,~] = EMGtoAct(optParams);
            a_l = a(1:(nTrials_l*nptsShort),:);
            a_r = a((nTrials_l*nptsShort + 1):end,:);
            % Divide each signal by std to get unitized variablity?
            for iii = 1:nMuscEmg
                a_l_unitVar(:,iii) = a_l(:,iii)/std(a_l(:,iii));
                a_r_unitVar(:,iii) = a_r(:,iii)/std(a_r(:,iii));
            end
            
            [synergyCommands_l,synergyWeights_l] = nnmf(a_l_unitVar,nSyn,'w0',synergyCommands_l,'h0',synergyWeights_l);
            [synergyCommands_r,synergyWeights_r] = nnmf(a_r_unitVar,nSyn,'w0',synergyCommands_r,'h0',synergyWeights_r);
            % Normalize by dividing commands by their max value and multiplying corresponding weight vectors by that same number
            for iii = 1:nSyn
                lMax = max(synergyCommands_l(:,iii));
                rMax = max(synergyCommands_r(:,iii));
                synergyCommandsNorm_l(:,iii) = synergyCommands_l(:,iii)/lMax;
                synergyWeightsNorm_l(iii,:) = synergyWeights_l(iii,:)*lMax;
                synergyCommandsNorm_r(:,iii) = synergyCommands_r(:,iii)/rMax;
                synergyWeightsNorm_r(iii,:) = synergyWeights_r(iii,:)*rMax;
            end
            
            optParams.a = a;
            optParams.synergyCommands_l = synergyCommandsNorm_l;
            optParams.synergyWeights_l = synergyWeightsNorm_l;
            optParams.synergyCommands_r = synergyCommandsNorm_r;
            optParams.synergyWeights_r = synergyWeightsNorm_r;
            
            % Set up variables and other parameters
            optParams.otherParams.lmoScale = guess.lmoScale;
            optParams.otherParams.ltsScale = guess.ltsScale;
            if GeometricAdjustments
                optParams.otherParams.coefsOffset = guess.coefsOffset;
            end
            
            xGuess1 = [guess.reconstSynWeights,guess.tDelay,guess.tAct,guess.aNonlin,guess.emgScale];
            xLowBounds1 = [lowBounds.reconstSynWeights,lowBounds.tDelay,lowBounds.tAct,lowBounds.aNonlin,lowBounds.emgScale];
            xUpBounds1 = [upBounds.reconstSynWeights,upBounds.tDelay,upBounds.tAct,upBounds.aNonlin,upBounds.emgScale];
            
            % Run optimization 1
            paramsTemp = fmincon(@costFunc1, xGuess1, [],[],[],[],...
                xLowBounds1, xUpBounds1,@nonlcon1,options,optParams);
            
            % Extract variables
            guess.reconstSynWeights = paramsTemp(1:(2*nSyn*(nMuscReconst-nReconstDup)));
            index = 2*nSyn*(nMuscReconst-nReconstDup);
            guess.tDelay = paramsTemp((index+1):(index+2));
            index = index + 2;
            guess.tAct = paramsTemp((index+1):(index+nMuscEmg));
            index = index + nMuscEmg;
            guess.aNonlin = paramsTemp((index+1):(index+nMuscEmg));
            index = index + nMuscEmg;
            guess.emgScale = paramsTemp((index+1):(index+2*nMuscEmg));

            
            % Step 2
            % Set up variables and other parameters
            optParams.otherParams.reconstSynWeights = guess.reconstSynWeights;
            optParams.otherParams.tDelay = guess.tDelay;
            optParams.otherParams.tAct = guess.tAct;
            optParams.otherParams.aNonlin = guess.aNonlin;
            optParams.otherParams.emgScale = guess.emgScale;
            
            xGuess2 = [guess.lmoScale,guess.ltsScale];
            xLowBounds2 = [lowBounds.lmoScale,lowBounds.ltsScale];
            xUpBounds2 = [upBounds.lmoScale,upBounds.ltsScale];
            
            % Run optimization 2
            paramsTemp = fmincon(@costFunc2, xGuess2, [],[],[],[],...
                xLowBounds2, xUpBounds2,@nonlcon2,options,optParams);
%             paramsTemp = fmincon(@costFunc3, xGuess3, [],[],[],[],...
%                 xLowBounds3, xUpBounds3,[],options,optParams);
            
            % Extract variables
            guess.lmoScale = paramsTemp(1:nMuscTotal);
            index = nMuscTotal;
            guess.ltsScale = paramsTemp((index+1):(index+nMuscTotal));
            
            
            if GeometricAdjustments
                % Step 3
                % Set up variables and other parameters
                optParams.otherParams.lmoScale = guess.lmoScale;
                optParams.otherParams.ltsScale = guess.ltsScale;
                
                xGuess3 = guess.coefsOffset;
                xLowBounds3 = lowBounds.coefsOffset;
                xUpBounds3 = upBounds.coefsOffset;
                
                % Run optimization 4
                paramsTemp = fmincon(@costFunc3, xGuess3, [],[],[],[],...
                    xLowBounds3, xUpBounds3,@nonlcon3,options,optParams);
%                 paramsTemp = fmincon(@costFunc4, xGuess4, [],[],[],[],...
%                     xLowBounds4, xUpBounds4,[],options,optParams);
                
                % Extract variables
                guess.coefsOffset = paramsTemp;
            end
        end
    end

    % Optimize just the reconstructed muscle synergies
    % Generate initial guess for the synergy commands and weights
    optParams.tDelay = guess.tDelay/10;
    optParams.tAct = guess.tAct/100;
    optParams.tDeact = 4*optParams.tAct;
    optParams.aNonlin = guess.aNonlin;
    optParams.emgScale = guess.emgScale;

    [a,~] = EMGtoAct(optParams);
    a_l = a(1:(nTrials_l*nptsShort),:);
    a_r = a((nTrials_l*nptsShort + 1):end,:);
    % Divide each signal by std to get unitized variablity?
    for iii = 1:nMuscEmg
        a_l_unitVar(:,iii) = a_l(:,iii)/std(a_l(:,iii));
        a_r_unitVar(:,iii) = a_r(:,iii)/std(a_r(:,iii));
    end

    [synergyCommands_l,synergyWeights_l] = nnmf(a_l_unitVar,nSyn,'w0',synergyCommands_l,'h0',synergyWeights_l);
    [synergyCommands_r,synergyWeights_r] = nnmf(a_r_unitVar,nSyn,'w0',synergyCommands_r,'h0',synergyWeights_r);
    % Normalize by dividing commands by their max value and multiplying corresponding weight vectors by that same number
    for iii = 1:nSyn
        lMax = max(synergyCommands_l(:,iii));
        rMax = max(synergyCommands_r(:,iii));
        synergyCommandsNorm_l(:,iii) = synergyCommands_l(:,iii)/lMax;
        synergyWeightsNorm_l(iii,:) = synergyWeights_l(iii,:)*lMax;
        synergyCommandsNorm_r(:,iii) = synergyCommands_r(:,iii)/rMax;
        synergyWeightsNorm_r(iii,:) = synergyWeights_r(iii,:)*rMax;
    end
    
    optParams.a = a;
    optParams.synergyCommands_l = synergyCommandsNorm_l;
    optParams.synergyWeights_l = synergyWeightsNorm_l;
    optParams.synergyCommands_r = synergyCommandsNorm_r;
    optParams.synergyWeights_r = synergyWeightsNorm_r;

    % Set up variables and other parameters
    optParams.otherParams.tDelay = guess.tDelay;
    optParams.otherParams.tAct = guess.tAct;
    optParams.otherParams.aNonlin = guess.aNonlin;
    optParams.otherParams.emgScale = guess.emgScale;
    optParams.otherParams.lmoScale = guess.lmoScale;
    optParams.otherParams.ltsScale = guess.ltsScale;
    if GeometricAdjustments
        optParams.otherParams.coefsOffset = guess.coefsOffset;
    end

    xGuess4 = guess.reconstSynWeights;
    xLowBounds4 = lowBounds.reconstSynWeights;
    xUpBounds4 = upBounds.reconstSynWeights;

    % Run optimization 4
    paramsTemp = fmincon(@costFunc4, xGuess4, [],[],[],[],...
        xLowBounds4, xUpBounds4,@nonlcon1,options,optParams);

    % Extract variables
    guess.reconstSynWeights = paramsTemp;

    % Run final optimization that includes all design variables
    % Set up variables and other parameters
    optParams.otherParams.reconstSynWeights = guess.reconstSynWeights;
    if GeometricAdjustments
%         xGuess = [guess.reconstSynWeights,guess.tDelay,guess.tAct,guess.aNonlin,...
%             guess.lmoScale,guess.ltsScale,guess.emgScale,guess.coefsOffset];
%         xLowBounds = [lowBounds.reconstSynWeights,lowBounds.tDelay,lowBounds.tAct,lowBounds.aNonlin,...
%             lowBounds.lmoScale,lowBounds.ltsScale,lowBounds.emgScale,lowBounds.coefsOffset];
%         xUpBounds = [upBounds.reconstSynWeights,upBounds.tDelay,upBounds.tAct,upBounds.aNonlin,...
%             upBounds.lmoScale,upBounds.ltsScale,upBounds.emgScale,upBounds.coefsOffset];
        xGuess = [guess.tDelay,guess.tAct,guess.aNonlin,guess.emgScale,...
            guess.lmoScale,guess.ltsScale,guess.coefsOffset];
        xLowBounds = [lowBounds.tDelay,lowBounds.tAct,lowBounds.aNonlin,lowBounds.emgScale,...
            lowBounds.lmoScale,lowBounds.ltsScale,lowBounds.coefsOffset];
        xUpBounds = [upBounds.tDelay,upBounds.tAct,upBounds.aNonlin,upBounds.emgScale,...
            upBounds.lmoScale,upBounds.ltsScale,upBounds.coefsOffset];
    else
%         xGuess = [guess.reconstSynWeights,guess.tDelay,guess.tAct,guess.aNonlin,...
%             guess.lmoScale,guess.ltsScale,guess.emgScale];
%         xLowBounds = [lowBounds.reconstSynWeights,lowBounds.tDelay,lowBounds.tAct,lowBounds.aNonlin,...
%             lowBounds.lmoScale,lowBounds.ltsScale,lowBounds.emgScale];
%         xUpBounds = [upBounds.reconstSynWeights,upBounds.tDelay,upBounds.tAct,upBounds.aNonlin,...
%             upBounds.lmoScale,upBounds.ltsScale,upBounds.emgScale];
        xGuess = [guess.tDelay,guess.tAct,guess.aNonlin,guess.emgScale,...
            guess.lmoScale,guess.ltsScale];
        xLowBounds = [lowBounds.tDelay,lowBounds.tAct,lowBounds.aNonlin,lowBounds.emgScale,...
            lowBounds.lmoScale,lowBounds.ltsScale];
        xUpBounds = [upBounds.tDelay,upBounds.tAct,upBounds.aNonlin,upBounds.emgScale,...
            upBounds.lmoScale,upBounds.ltsScale];
    end
    
    % Run optimization
    [finalParams, FVAL] = fmincon(@costFunc, xGuess, [],[],[],[],...
        xLowBounds, xUpBounds,@nonlcon,optionsLarge,optParams);
%     [finalParams, FVAL] = fmincon(@costFunc, xGuess, [],[],[],[],...
%         xLowBounds, xUpBounds,[],optionsLarge,optParams);
    
    BestOpt = FVAL;
    params = [guess.reconstSynWeights,finalParams];
    
else
    if GeometricAdjustments
        finalParams = [guess.tDelay,guess.tAct,guess.aNonlin,guess.emgScale,...
            guess.lmoScale,guess.ltsScale,guess.coefsOffset];
    else
        finalParams = [guess.tDelay,guess.tAct,guess.aNonlin,guess.emgScale,...
            guess.lmoScale,guess.ltsScale];
    end
    
    params = [guess.reconstSynWeights,finalParams];
end


%% Save Data

if SaveData
    save(SaveNameParams, 'params');
    
end


%% Extract new parameter values

% extract values from parameters matrix
reconstSynWeights = params(1:(2*nSyn*(nMuscReconst-nReconstDup)));
index = 2*nSyn*(nMuscReconst-nReconstDup);
tDelay = params(1,index+1:index+2)/10;
index = index + 2;
tAct = params(1,index+1:index+nMuscEmg)/100;
index = index + nMuscEmg;
tDeact = 4*tAct;
aNonlinOpt = params(1,index+1:index+nMuscEmg);
index = index + nMuscEmg;
emgScaleOpt = params(1,(index+1):(index+2*nMuscEmg));
index = index + 2*nMuscEmg;
VmaxFactor = 10;
lmoScaleOpt = params(1,(index+1):(index+nMuscTotal));
index = index + nMuscTotal;
ltsScaleOpt = params(1,(index+1):(index+nMuscTotal));
index = index + nMuscTotal;
if GeometricAdjustments
    coefsOffsetOpt = params(1,(index+1):end).*optParams.coefsScaleFactors;
else
    coefsOffsetOpt = 0;
end

if 1
% Determine appropriate scale factors for terms in optimization
coefsScaleFactors = exp(ceil(log10(abs(coefsOffsetOpt)))+1);
save muscleGeoCoefScaleFactors.mat coefsScaleFactors
end

% adjust old muscle parameters values with optimized scale factors/offsets
coefsNew = coefsOrig+coefsOffsetOpt';
ltsOpt = optParams.lts.*ltsScaleOpt;
lmoOpt = optParams.lmo.*lmoScaleOpt;

% add new parameters to optimization structure
optParams.reconsSynWeights = reconstSynWeights;
optParams.VmaxFactor = VmaxFactor;
optParams.lmoScale = lmoScaleOpt;
optParams.ltsScale = ltsScaleOpt;
optParams.emgScale = emgScaleOpt;
optParams.tDelay = tDelay;
optParams.tAct = tAct;
optParams.tDeact = tDeact;
optParams.aNonlin = aNonlinOpt;


%% Make design variable value plots

% make plots of parameter changes/values
if makeParameterChangePlotsSwitch
    makeParameterChangePlots(optParams);
end


%% Calculate new moments and muscle parameter values

[a, Excitations] = EMGtoAct(optParams);

if SaveData
    save(SaveNameSynergies,'synergyCommands_l','synergyCommands_r','synergyWeights_l','synergyWeights_r','synergyCommandsNorm_l','synergyCommandsNorm_r','synergyWeightsNorm_l','synergyWeightsNorm_r');
end
synWeightsTemp = reconstSynWeights(1,1:(nSyn*(nMuscReconst-nReconstDup)));
reconstSynWeights_l = reshape(synWeightsTemp,[nSyn,(nMuscReconst-nReconstDup)]);
synWeightsTemp = reconstSynWeights(1,(nSyn*(nMuscReconst-nReconstDup)+1):(2*nSyn*(nMuscReconst-nReconstDup)));
reconstSynWeights_r = reshape(synWeightsTemp,[nSyn,(nMuscReconst-nReconstDup)]);
unknownActs_l = synergyCommandsNorm_l*reconstSynWeights_l;
unknownActs_r = synergyCommandsNorm_r*reconstSynWeights_r;
unknownActs = [unknownActs_l;unknownActs_r];

a = [a,unknownActs(1:size(a,1),1),unknownActs(1:size(a,1),:)]; % NOTE: this part is specific to the muscles and EMG you have

optParams.a = a;
optParams.lts = ltsOpt;
optParams.lmo = lmoOpt;
optParams.coefs = coefsNew;

[moments, lmtilda, vmtilda, HipFEMA, HipAAMA, KneeMA, AnkleMA, SubtalarMA, ...
    muscMoments, passiveF, Lmt, muscleForce] = calcMoments(optParams,1);

% Calculate passive forces at each joint during gait
PassiveHipFEM = sum(passiveF.*HipFEMA,2);
PassiveHipAAM = sum(passiveF.*HipAAMA,2);
PassiveKneeFEM = sum(passiveF.*KneeMA,2);
PassiveAnklePFM = sum(passiveF.*AnkleMA,2);
PassiveAnkleIEM = sum(passiveF.*SubtalarMA,2);

PassiveMGait = [PassiveHipFEM,PassiveHipAAM,PassiveKneeFEM,PassiveAnklePFM,PassiveAnkleIEM];
PassiveMGait = reshape(PassiveMGait, nptsShort,nTrials,nJoints);

optParams.PassiveMGait = PassiveMGait;

% Make Synergy Commands Plots
if makeSynergyCommandPlotsSwitch
    makeSynergyCommandPlots(synergyCommandsNorm_l,synergyCommandsNorm_r,optParams);
end

%% Calculate errors between model and inverse dynamics

% Calculate RMS errors
HipRMS = sqrt(sum((moments(:,1)-optParams.IDloads(:,1)).^2)/(optParams.nframesAll-1));
HipAARMS = sqrt(sum((moments(:,2)-optParams.IDloads(:,2)).^2)/(optParams.nframesAll-1));
KneeRMS = sqrt(sum((moments(:,3)-optParams.IDloads(:,3)).^2)/(optParams.nframesAll-1));
AnkleRMS = sqrt(sum((moments(:,4)-optParams.IDloads(:,4)).^2)/(optParams.nframesAll-1));
SubtalarRMS = sqrt(sum((moments(:,5)-optParams.IDloads(:,5)).^2)/(optParams.nframesAll-1));

% combine errros into single matrix
RMS = [HipRMS HipAARMS KneeRMS AnkleRMS SubtalarRMS]

% Calculate mean absolute errors
HipMAE = sum(abs(moments(:,1)-optParams.IDloads(:,1))/optParams.nframesAll);
HipAAMAE = sum(abs(moments(:,2)-optParams.IDloads(:,2))/optParams.nframesAll);
KneeMAE = sum(abs(moments(:,3)-optParams.IDloads(:,3))/optParams.nframesAll);
AnkleMAE = sum(abs(moments(:,4)-optParams.IDloads(:,4))/optParams.nframesAll);
SubtalarMAE = sum(abs(moments(:,5)-optParams.IDloads(:,5))/optParams.nframesAll);

% combine errros into single matrix
MAEs = [HipMAE HipAAMAE KneeMAE AnkleMAE SubtalarMAE]

% reshape/permute moments and IDloads into form (time
% frames)/(trials)/(joint)
moments = reshape(moments,nptsShort,nTrials,5);
IDloads = permute(IDloads, [1 3 2]);

% Calculate median errors instead of errors over all trials
HipMAEMed = median(sum(abs(moments(:,:,1)-IDloads(:,:,1)))/nptsShort);
HipAAMAEMed = median(sum(abs(moments(:,:,2)-IDloads(:,:,2)))/nptsShort);
KneeMAEMed = median(sum(abs(moments(:,:,3)-IDloads(:,:,3)))/nptsShort);
AnkleMAEMed = median(sum(abs(moments(:,:,4)-IDloads(:,:,4)))/nptsShort);
SubtalarMAEMed = median(sum(abs(moments(:,:,5)-IDloads(:,:,5)))/nptsShort);

% combine errros into single matrix
MAEMed = [HipMAEMed HipAAMAEMed KneeMAEMed AnkleMAEMed SubtalarMAEMed]

% Calculate max and min values for lmtilda and vmtilda for inspection
maxlmtilda = max(lmtilda)
minlmtilda = min(lmtilda)
maxvmtilda = max(vmtilda)
minvmtilda = min(vmtilda)


%% Make moment matching plots
% for plotting individual muscle contributions to moments
if makeIndividualMuscMomentPlotsSwitch
    makeIndividualMuscMomentPlots(moments,muscMoments,optParams);
end

if makeMomentMatchingPlotsSwitch
    makeMomentMatchingPlots(moments,IDloads,optParams);
end


%% Reshape data into form (time frames)/(trials)/(joint or muscle)
muscMoments = reshape(muscMoments, nptsShort,nTrials,nMuscTotal,5);
passiveF = reshape(passiveF,nptsShort,nTrials,nMuscTotal);
Lmt = reshape(Lmt,nptsShort,nTrials,nMuscTotal);
muscleForce = reshape(muscleForce,nptsShort,nTrials,nMuscTotal);

Lmt = reshape(Lmt,nptsShort,nTrials,nMuscTotal);
lmtilda = reshape(lmtilda,nptsShort,nTrials,nMuscTotal);
vmtilda = reshape(vmtilda,nptsShort,nTrials,nMuscTotal);
HipFEMA = reshape(HipFEMA,nptsShort,nTrials,nMuscTotal);
HipAAMA = reshape(HipAAMA,nptsShort,nTrials,nMuscTotal);
KneeMA = reshape(KneeMA,nptsShort,nTrials,nMuscTotal);
AnkleMA = reshape(AnkleMA,nptsShort,nTrials,nMuscTotal);
SubtalarMA = reshape(SubtalarMA,nptsShort,nTrials,nMuscTotal);
OpensimMuscleLabels = optParams.MuscLabels;


%% Make muscle parameter (lmtilda lmt etc...) plots
if makeMuscleParameterPlotsSwitch
    
    makeMuscleParameterPlots(Lmt,lmtilda,vmtilda,HipFEMA,HipAAMA,KneeMA,AnkleMA,...
        SubtalarMA,Lmtorig,HipMAorig,HipAAMAorig,KneeMAorig,AnkleMAorig,SubtalarMAorig,optParams)
    
end


%% Make muscle excitation plots
if makeExcitationsPlotsSwitch
    makeExcitationPlots(Excitations,a,optParams);
end

if makeEmgPlotsSwitch
    makeEmgPlots(emgAll,optParams);
end

%% Make passive moment matching plots
if makePassiveMomentMatchingPlotsSwitch
    makePassiveMomentMatchingPlots(optParams);
end

%% Make joint angle plots
if makeJointAnglePlotsSwitch
    makeJointAnglePlots(optParams);
end



%% Calculate R values between moment arms and lmt for each muscle
Lmtorigstacked = reshape(permute(Lmtorig,[1 3 2]),nFrames,nMuscTotal);
Lmtstacked = reshape(permute(Lmt,[1 2 3]),nFrames,nMuscTotal);
HipMAorigstacked = reshape(permute(HipMAorig,[1 3 2]),nFrames,nMuscTotal);
HipMAstacked = reshape(permute(HipFEMA,[1 2 3]),nFrames,nMuscTotal);
HipAAMAorigstacked = reshape(permute(HipAAMAorig,[1 3 2]),nFrames,nMuscTotal);
HipAAMAstacked = reshape(permute(HipAAMA,[1 2 3]),nFrames,nMuscTotal);
KneeMAorigstacked = reshape(permute(KneeMAorig,[1 3 2]),nFrames,nMuscTotal);
KneeMAstacked = reshape(permute(KneeMA,[1 2 3]),nFrames,nMuscTotal);
AnkleMAorigstacked = reshape(permute(AnkleMAorig,[1 3 2]),nFrames,nMuscTotal);
AnkleMAstacked = reshape(permute(AnkleMA,[1 2 3]),nFrames,nMuscTotal);
SubtalarMAorigstacked = reshape(permute(SubtalarMAorig,[1 3 2]),nFrames,nMuscTotal);
SubtalarMAstacked = reshape(permute(SubtalarMA,[1 2 3]),nFrames,nMuscTotal);

for iii = 1:nMuscTotal
    R = corrcoef(Lmtstacked(:,iii),Lmtorigstacked(:,iii));
    RLmt(iii) = R(2);
    R = corrcoef(HipMAstacked(:,iii),HipMAorigstacked(:,iii));
    RHip(iii) = R(2);
    R = corrcoef(HipAAMAstacked(:,iii),HipAAMAorigstacked(:,iii));
    RHipAA(iii) = R(2);
    R = corrcoef(KneeMAstacked(:,iii),KneeMAorigstacked(:,iii));
    RKnee(iii) = R(2);
    R = corrcoef(AnkleMAstacked(:,iii),AnkleMAorigstacked(:,iii));
    RAnkle(iii) = R(2);
    R = corrcoef(SubtalarMAstacked(:,iii),SubtalarMAorigstacked(:,iii));
    RSubtalar(iii) = R(2);
end


%% Calculate changes in moment arm values
HipMAorig = permute(HipMAorig, [1 3 2]);
HipAAMAorig = permute(HipAAMAorig, [1 3 2]);
KneeMAorig = permute(KneeMAorig, [1 3 2]);
AnkleMAorig = permute(AnkleMAorig, [1 3 2]);
SubtalarMAorig = permute(SubtalarMAorig, [1 3 2]);

[HipMADiff, index] = max(max(max(abs((HipFEMA-HipMAorig)))));
fprintf(['\nThe ' OpensimMuscleLabels{index} ' hip FE moment arm max change was ' num2str(HipMADiff) '.'])
[HipMADiff, index] = max(mean(mean(abs((HipFEMA-HipMAorig)))));
fprintf(['\nThe ' OpensimMuscleLabels{index} ' hip FE moment arm mean change was ' num2str(HipMADiff) '.\n'])

[HipAAMADiff, index] = max(max(max((HipAAMA-HipAAMAorig))));
fprintf(['\nThe ' OpensimMuscleLabels{index} ' hip AA moment arm max change was ' num2str(HipAAMADiff) '.'])
[HipAAMADiff, index] = max(mean(mean(abs((HipAAMA-HipAAMAorig)))));
fprintf(['\nThe ' OpensimMuscleLabels{index} ' hip AA moment arm mean change was ' num2str(HipAAMADiff) '.\n'])

[KneeMADiff, index] = max(max(max(abs((KneeMA-KneeMAorig)))));
fprintf(['\nThe ' OpensimMuscleLabels{index} ' knee FE moment arm max change was ' num2str(KneeMADiff) '.'])
[KneeMADiff, index] = max(mean(mean(abs((KneeMA-KneeMAorig)))));
fprintf(['\nThe ' OpensimMuscleLabels{index} ' knee FE moment arm mean change was ' num2str(KneeMADiff) '.\n'])

[AnkleMADiff, index] = max(max(max(abs((AnkleMA-AnkleMAorig)))));
fprintf(['\nThe ' OpensimMuscleLabels{index} ' ankle PF moment arm max change was ' num2str(AnkleMADiff) '.'])
[AnkleMADiff, index] = max(mean(mean(abs((AnkleMA-AnkleMAorig)))));
fprintf(['\nThe ' OpensimMuscleLabels{index} ' ankle PF moment arm mean change was ' num2str(AnkleMADiff) '.\n'])

[SubtalarMADiff, index] = max(max(max(abs((SubtalarMA-SubtalarMAorig)))));
fprintf(['\nThe ' OpensimMuscleLabels{index} ' subtalar IE moment arm max change was ' num2str(SubtalarMADiff) '.'])
[SubtalarMADiff, index] = max(mean(mean(abs((SubtalarMA-SubtalarMAorig)))));
fprintf(['\nThe ' OpensimMuscleLabels{index} ' subtalar IE moment arm mean change was ' num2str(SubtalarMADiff) '.\n'])

Lmtorig = permute(Lmtorig, [1 3 2]);
HipMAorig = permute(HipMAorig, [1 3 2]);
HipAAMAorig = permute(HipAAMAorig, [1 3 2]);
KneeMAorig = permute(KneeMAorig, [1 3 2]);
AnkleMAorig = permute(AnkleMAorig, [1 3 2]);
SubtalarMAorig = permute(SubtalarMAorig, [1 3 2]);

OpensimMuscleLabels = optParams.MuscLabels


%% Save all results
Fmax = (optParams.Vmusc./lmoOpt)*optParams.sigma;

if SaveData
    save(SaveNameResults, 'tDelay', 'tAct', 'aNonlinOpt', 'lmoScaleOpt', 'ltsScaleOpt',...
        'emgScaleOpt', 'OpensimMuscleLabels', 'Lmtorig', 'HipMAorig', 'HipAAMAorig',...
        'KneeMAorig', 'AnkleMAorig', 'SubtalarMAorig', 'moments', 'IDloads', 'lmtilda',...
        'vmtilda', 'HipFEMA', 'HipAAMA', 'KneeMA', 'AnkleMA', 'SubtalarMA', 'muscMoments',...
        'passiveF', 'Lmt', 'muscleForce', 'JAngles','a','Excitations','Fmax','alpha');
end


%% Extra plots that may be useful but do not plot by default
if 0
    
    a = reshape(a,nptsShort,nTrials,nMuscTotal);
    AnkleMAorig = permute(AnkleMAorig,[1 3 2]);
    for iii = [25:27 31]
        figure
%         keyboard;
        subplot(6,1,1), plot(0:optParams.SampleStep:100,mean(emgAll(21:121,iii,1:nTrials_l),3),'b','LineWidth',2)
        hold on
        subplot(6,1,1), plot(0:optParams.SampleStep:100,mean(Excitations(:,1:nTrials_l,iii),2),'r','LineWidth',2)
        axis([0 100 0 0.5])
        title(['LEFT ' OpensimMuscleLabels{iii} ' EMG (blue) and Excitation (red)'])
        
        subplot(6,1,2), plot(0:optParams.SampleStep:100,mean(a(:,1:nTrials_l,iii),2),'b','LineWidth',2)
        axis([0 100 0 0.5])
        title('Activation')
        
        subplot(6,1,3), plot(0:optParams.SampleStep:100,mean(muscleForce(:,1:nTrials_l,iii),2),'k','LineWidth',2)
        hold on
        subplot(6,1,3), plot(0:optParams.SampleStep:100,mean(passiveF(:,1:nTrials_l,iii),2),'r','LineWidth',2)
        subplot(6,1,3), plot(0:optParams.SampleStep:100,mean(muscleForce(:,1:nTrials_l,iii)-passiveF(:,1:nTrials_l,iii),2),'b','LineWidth',2)
        %         axis([0 100 0 500])
        title('Muscle Forces: black total, blue active, red passive')
        
        subplot(6,1,4), plot(0:optParams.SampleStep:100,-mean(AnkleMA(:,1:nTrials_l,iii),2),'r','LineWidth',2)
        hold on
        subplot(6,1,4), plot(0:optParams.SampleStep:100,-mean(AnkleMAorig(:,1:nTrials_l,iii),2),'b','LineWidth',2)
        axis([0 100 0.03 0.06])
        title('Ankle Moment Arms: blue original, red optimized')
        
        subplot(6,1,5), plot(0:optParams.SampleStep:100,-mean(AnkleMA(:,1:nTrials_l,iii).*muscleForce(:,1:nTrials_l,iii),2),'k','LineWidth',2)
        hold on
        subplot(6,1,5), plot(0:optParams.SampleStep:100,-mean(AnkleMA(:,1:nTrials_l,iii).*passiveF(:,1:nTrials_l,iii),2),'r','LineWidth',2)
        subplot(6,1,5), plot(0:optParams.SampleStep:100,-mean(AnkleMA(:,1:nTrials_l,iii).*(muscleForce(:,1:nTrials_l,iii)-passiveF(:,1:nTrials_l,iii)),2),'b','LineWidth',2)
        subplot(6,1,5), plot(0:optParams.SampleStep:100,-mean(AnkleMAorig(:,1:nTrials_l,iii).*muscleForce(:,1:nTrials_l,iii),2),'k.-','LineWidth',2)
        subplot(6,1,5), plot(0:optParams.SampleStep:100,-mean(AnkleMAorig(:,1:nTrials_l,iii).*passiveF(:,1:nTrials_l,iii),2),'r.-','LineWidth',2)
        subplot(6,1,5), plot(0:optParams.SampleStep:100,-mean(AnkleMAorig(:,1:nTrials_l,iii).*(muscleForce(:,1:nTrials_l,iii)-passiveF(:,1:nTrials_l,iii)),2),'b.-','LineWidth',2)
        %         axis([0 100 0 25])
        title('Muscle Moments: black total, blue active, red passive, dashed are with original moment arm')
        
        subplot(6,1,6), plot(0:optParams.SampleStep:100,-mean(IDloads(:,1:nTrials_l,4),2),'k','LineWidth',2)
        axis([0 100 0 100])
        title('ID Load')
        
        figure
        subplot(6,1,1), plot(0:optParams.SampleStep:100,mean(emgAll(21:121,iii,nTrials_l+1:end),3),'b','LineWidth',2)
        hold on
        subplot(6,1,1), plot(0:optParams.SampleStep:100,mean(Excitations(:,nTrials_l+1:end,iii),2),'r','LineWidth',2)
        axis([0 100 0 0.5])
        title(['Right ' OpensimMuscleLabels{iii} ' EMG (blue) and Excitation (red)'])
        
        subplot(6,1,2), plot(0:optParams.SampleStep:100,mean(a(:,nTrials_l+1:end,iii),2),'b','LineWidth',2)
        axis([0 100 0 0.5])
        title('Activation')
        
        subplot(6,1,3), plot(0:optParams.SampleStep:100,mean(muscleForce(:,nTrials_l+1:end,iii),2),'k','LineWidth',2)
        hold on
        subplot(6,1,3), plot(0:optParams.SampleStep:100,mean(passiveF(:,nTrials_l+1:end,iii),2),'r','LineWidth',2)
        subplot(6,1,3), plot(0:optParams.SampleStep:100,mean(muscleForce(:,nTrials_l+1:end,iii)-passiveF(:,nTrials_l+1:end,iii),2),'b','LineWidth',2)
        %         axis([0 100 0 500])
        title('Muscle Forces: black total, blue active, red passive')
        
        subplot(6,1,4), plot(0:optParams.SampleStep:100,-mean(AnkleMA(:,nTrials_l+1:end,iii),2),'r','LineWidth',2)
        hold on
        subplot(6,1,4), plot(0:optParams.SampleStep:100,-mean(AnkleMAorig(:,nTrials_l+1:end,iii),2),'b','LineWidth',2)
        axis([0 100 0.03 0.06])
        title('Ankle Moment Arms: blue original, red optimized')
        
        subplot(6,1,5), plot(0:optParams.SampleStep:100,-mean(AnkleMA(:,nTrials_l+1:end,iii).*muscleForce(:,nTrials_l+1:end,iii),2),'k','LineWidth',2)
        hold on
        subplot(6,1,5), plot(0:optParams.SampleStep:100,-mean(AnkleMA(:,nTrials_l+1:end,iii).*passiveF(:,nTrials_l+1:end,iii),2),'r','LineWidth',2)
        subplot(6,1,5), plot(0:optParams.SampleStep:100,-mean(AnkleMA(:,nTrials_l+1:end,iii).*(muscleForce(:,nTrials_l+1:end,iii)-passiveF(:,nTrials_l+1:end,iii)),2),'b','LineWidth',2)
        subplot(6,1,5), plot(0:optParams.SampleStep:100,-mean(AnkleMAorig(:,nTrials_l+1:end,iii).*muscleForce(:,nTrials_l+1:end,iii),2),'k.-','LineWidth',2)
        subplot(6,1,5), plot(0:optParams.SampleStep:100,-mean(AnkleMAorig(:,nTrials_l+1:end,iii).*passiveF(:,nTrials_l+1:end,iii),2),'r.-','LineWidth',2)
        subplot(6,1,5), plot(0:optParams.SampleStep:100,-mean(AnkleMAorig(:,nTrials_l+1:end,iii).*(muscleForce(:,nTrials_l+1:end,iii)-passiveF(:,nTrials_l+1:end,iii)),2),'b.-','LineWidth',2)
        %         axis([0 100 0 25])
        title('Muscle Moments: black total, blue active, red passive, dashed are with original moment arm')
        
        subplot(6,1,6), plot(0:optParams.SampleStep:100,-mean(IDloads(:,nTrials_l+1:end,4),2),'k','LineWidth',2)
        axis([0 100 0 100])
        title('ID Load')
    end
    
    figure
    for iii = 1:31
        subplot(6,6,iii), plot(0:optParams.SampleStep:100,Excitations(:,1:nTrials_l,iii),'LineWidth',1);
        axis([0 100 0 1])
        title(OpensimMuscleLabels{iii});
    end
end



end

%% Function Definitions
function emgFiltered = processEMG(emgData, dt, HPcutoff, LPcutoff)
% Function that process EMG data according the the process described by
% Lloyd and Besier (2003)
% Inputs:
%       emgData is a M x N matrix of unfiltered EMG data where M is the
%           number of time frames and N is the number of muscles
%       dt is the time step used in the EMG data
%       HPcutoff is the cutoff frequency for the high pass filter
%       LPcutoff is the cutoff frequency for the low pass filter
% Outputs:
%       emgFiltered is the filtered EMG data

SampRate = 1/mean(dt);

% High pass filter the data
degree = 4;
[b,a] = butter(degree, 2*HPcutoff/SampRate,'high');
% [sos,g] = zp2sos(z,p,k);      % Convert to SOS(second-order-sections) form
EMG_HP = filtfilt(b,a,emgData);

% Demean
EMG_DM = EMG_HP-ones(size(EMG_HP,1),1)*mean(EMG_HP);

% Rectify
EMG_RF = abs(EMG_DM);

% EMG_RF = hampel(EMG_RF,100);

% Low pass filter
% degree = 4;
% LPcutoff = 6;


[b,a] = butter(degree,2*LPcutoff/SampRate);
% [sos,g] = zp2sos(z,p,k);      % Convert to SOS(second-order-sections) form
% pad data with final values
% numPad = 500;
% EMG_Padded = [ones(numPad,1)*EMG_RF(1,:);EMG_RF;ones(numPad,1)*EMG_RF(end,:)];
EMG_LP = filtfilt(b,a,EMG_RF);
emgFiltered = EMG_LP;%(numPad+1:end-numPad,:);

% emgFiltered = EMG_medFilt100;
% 
% close all
% for i = 1:16
%    figure
%    plot(emgFiltered(:,i),'b','LineWidth',1.5)
%    hold on
%    plot(EMG_medFilt10(:,i),'r','LineWidth',1.5)
%    plot(EMG_medFilt25(:,i),'g','LineWidth',1.5)
%    plot(EMG_medFilt50(:,i),'c','LineWidth',1.5)
%    plot(EMG_medFilt100(:,i),'k','LineWidth',1.5)
%    plot(emgFiltered_medFilt(:,i),'m','LineWidth',1.5)
% end

% keyboard

% Remove any negative EMG values that may still exist
emgFiltered(emgFiltered<0) = 0;
end

function [Mat] = buildMuscleMatrices(JAngles,JVels)
% This function creates matrices that when multiplied by the muscle
% geometry coefficients will produce the Lmt, Vmt and moment arm curves
% each matrix has the form [Angle Angle^2 Angle^3 ... other joint ...
% interaction terms]

nFrames = size(JAngles,1);
onesCol = ones(nFrames,1);
zerosMat = zeros(nFrames,3);

% matrices for determining lmt
HipFEMat = [JAngles(:,1) JAngles(:,1).^2 JAngles(:,1).^3];
HipAAMat = [JAngles(:,2) JAngles(:,2).^2 JAngles(:,2).^3];
HipIEMat = [JAngles(:,6) JAngles(:,6).^2 JAngles(:,6).^3];
InteractionFEAA = [JAngles(:,1).*JAngles(:,2) JAngles(:,1).^2.*JAngles(:,2) JAngles(:,1).*JAngles(:,2).^2];
InteractionIEAA = [JAngles(:,6).*JAngles(:,2) JAngles(:,6).^2.*JAngles(:,2) JAngles(:,6).*JAngles(:,2).^2];
InteractionFEIE = [JAngles(:,1).*JAngles(:,6) JAngles(:,1).^2.*JAngles(:,6) JAngles(:,1).*JAngles(:,6).^2];
KneeMat = [JAngles(:,3) JAngles(:,3).^2 JAngles(:,3).^3];
AnkleMat = [JAngles(:,4) JAngles(:,4).^2 JAngles(:,4).^3];
SubtalarMat = [JAngles(:,5) JAngles(:,5).^2 JAngles(:,5).^3];
InteractionAnkleSub = [JAngles(:,4).*JAngles(:,5) JAngles(:,4).^2.*JAngles(:,5) JAngles(:,4).*JAngles(:,5).^2];

% matrices for determining vmt
HipFEVelMat = [JVels(:,1) 2*JVels(:,1).*JAngles(:,1) 3*JVels(:,1).*JAngles(:,1).^2];
HipAAVelMat = [JVels(:,2) 2*JVels(:,2).*JAngles(:,2) 3*JVels(:,2).*JAngles(:,2).^2];
HipIEVelMat = [JVels(:,6) 2*JVels(:,6).*JAngles(:,6) 3*JVels(:,6).*JAngles(:,6).^2];
InteractionFEAAVel = [JVels(:,1).*JAngles(:,2)+JAngles(:,1).*JVels(:,2) 2*JAngles(:,1).*JVels(:,1).*JAngles(:,2)+JAngles(:,1).^2.*JVels(:,2) JVels(:,1).*JAngles(:,2).^2+2*JAngles(:,1).*JVels(:,2).*JAngles(:,2)];
InteractionIEAAVel = [JVels(:,6).*JAngles(:,2)+JAngles(:,6).*JVels(:,2) 2*JAngles(:,6).*JVels(:,6).*JAngles(:,2)+JAngles(:,6).^2.*JVels(:,2) JVels(:,6).*JAngles(:,2).^2+2*JAngles(:,6).*JVels(:,2).*JAngles(:,2)];
InteractionFEIEVel = [JVels(:,1).*JAngles(:,6)+JAngles(:,1).*JVels(:,6) 2*JAngles(:,1).*JVels(:,1).*JAngles(:,6)+JAngles(:,1).^2.*JVels(:,6) JVels(:,1).*JAngles(:,6).^2+2*JAngles(:,1).*JVels(:,6).*JAngles(:,6)];
KneeVelMat = [JVels(:,3) 2*JVels(:,3).*JAngles(:,3) 3*JVels(:,3).*JAngles(:,3).^2];
AnkleVelMat = [JVels(:,4) 2*JVels(:,4).*JAngles(:,4) 3*JVels(:,4).*JAngles(:,4).^2];
SubtalarVelMat = [JVels(:,5) 2*JVels(:,5).*JAngles(:,5) 3*JVels(:,5).*JAngles(:,5).^2];
InteractionAnkleSubVel = [JVels(:,4).*JAngles(:,5)+JAngles(:,4).*JVels(:,5) 2*JAngles(:,4).*JVels(:,4).*JAngles(:,5)+JAngles(:,4).^2.*JVels(:,5) JVels(:,4).*JAngles(:,5).^2+2*JAngles(:,4).*JVels(:,5).*JAngles(:,5)];

% matrices for determining moment arms
HipFEMAMat = -[onesCol 2*JAngles(:,1) 3*JAngles(:,1).^2];
HipAAMAMat = -[onesCol 2*JAngles(:,2) 3*JAngles(:,2).^2];
InteractionFEAAMA_FE = -[JAngles(:,2) 2*JAngles(:,1).*JAngles(:,2) JAngles(:,2).^2];
InteractionFEAAMA_AA = -[JAngles(:,1) JAngles(:,1).^2 2*JAngles(:,1).*JAngles(:,2)];
InteractionFEIEMA_FE = -[JAngles(:,6) 2*JAngles(:,1).*JAngles(:,6) JAngles(:,6).^2];
InteractionIEAAMA_AA = -[JAngles(:,6) JAngles(:,6).^2 2*JAngles(:,6).*JAngles(:,2)];
KneeMAMat = -[onesCol 2*JAngles(:,3) 3*JAngles(:,3).^2];
AnkleMAMat = -[onesCol 2*JAngles(:,4) 3*JAngles(:,4).^2];
SubtalarMAMat = -[onesCol 2*JAngles(:,5) 3*JAngles(:,5).^2];
InteractionAnkleMA = -[JAngles(:,5) 2*JAngles(:,4).*JAngles(:,5) JAngles(:,5).^2];
InteractionSubMA = -[JAngles(:,4) JAngles(:,4).^2 2*JAngles(:,4).*JAngles(:,5)];

% combine all matrices into a structure
Mat{1} = [onesCol HipFEMat HipAAMat HipIEMat InteractionFEAA InteractionIEAA InteractionFEIE;...
    0*onesCol HipFEVelMat HipAAVelMat HipIEVelMat InteractionFEAAVel InteractionIEAAVel InteractionFEIEVel;...
    0*onesCol HipFEMAMat zerosMat zerosMat InteractionFEAAMA_FE zerosMat InteractionFEIEMA_FE;...
    0*onesCol zerosMat HipAAMAMat zerosMat InteractionFEAAMA_AA InteractionIEAAMA_AA zerosMat];
Mat{2} = [onesCol HipFEMat HipAAMat HipIEMat KneeMat InteractionFEAA InteractionIEAA InteractionFEIE;...
    0*onesCol HipFEVelMat HipAAVelMat HipIEVelMat KneeVelMat InteractionFEAAVel InteractionIEAAVel InteractionFEIEVel;...
    0*onesCol HipFEMAMat zerosMat zerosMat zerosMat InteractionFEAAMA_FE zerosMat InteractionFEIEMA_FE;...
    0*onesCol zerosMat HipAAMAMat zerosMat zerosMat InteractionFEAAMA_AA InteractionIEAAMA_AA zerosMat;...
    0*onesCol zerosMat zerosMat zerosMat KneeMAMat zerosMat zerosMat zerosMat];
Mat{3} = [onesCol KneeMat;...
    0*onesCol KneeVelMat;...
    0*onesCol KneeMAMat];
Mat{4} = [onesCol KneeMat AnkleMat SubtalarMat InteractionAnkleSub;...
    0*onesCol KneeVelMat AnkleVelMat SubtalarVelMat InteractionAnkleSubVel;...
    0*onesCol KneeMAMat zerosMat zerosMat zerosMat;...
    0*onesCol zerosMat AnkleMAMat zerosMat InteractionAnkleMA;...
    0*onesCol zerosMat zerosMat SubtalarMAMat InteractionSubMA];
Mat{5} = [onesCol AnkleMat SubtalarMat InteractionAnkleSub;...
    0*onesCol AnkleVelMat SubtalarVelMat InteractionAnkleSubVel;...
    0*onesCol AnkleMAMat zerosMat InteractionAnkleMA;...
    0*onesCol zerosMat SubtalarMAMat InteractionSubMA];
end

function [a, Excitations] = EMGtoAct(optParams)

nMuscEmg = optParams.nMuscEmg;
nTrials = optParams.nTrials;
Time = optParams.Time;
tDelay = optParams.tDelay;
EMGsplines = optParams.EMGsplines;
tAct(1,1,:) = optParams.tAct;
tDeact(1,1,:) = optParams.tDeact;
aNonlin = optParams.aNonlin;
emgScale = optParams.emgScale;
onesCol = optParams.onesCol;
nptsShort = optParams.nptsShort;
nptsLong = optParams.nptsLong;
SampleStep = optParams.SampleStep;

% subplot(1,5,1)
% hold on
% plot(0:100,optParams.EMG(21:121,50,24),'k','LineWidth',3)
% set(gca,'FontSize',24,'xtick',[0 100],'ytick',[0 0.5 1])
% xlabel('% Gait Cycle')
% title('EMG Envelope')
% axis([0 100 0 1])

interpStep = 1;
nptsInterp = interpStep*(nptsLong-1)+1;

TimeInterp = linspace(0,1,(size(Time,1)-1)*interpStep+1)';

EMG = zeros(nptsInterp,nMuscEmg,nTrials);
for j = 1:optParams.nTrials_l

    EMG(:,:,j) = ppval((Time(end,j)-Time(1,j))*TimeInterp+Time(1,j)-tDelay(1),EMGsplines{j})';
    
end
for j = (optParams.nTrials_l+1):(optParams.nTrials_l+optParams.nTrials_r)
    
    EMG(:,:,j) = ppval((Time(end,j)-Time(1,j))*TimeInterp+Time(1,j)-tDelay(2),EMGsplines{j})';
    
end

EMG = permute(EMG,[1 3 2]);

% subplot(1,5,2)
% hold on
% plot(0:100,EMG(21:121,50,24),'k','LineWidth',3)
% set(gca,'FontSize',24,'xtick',[0 100],'ytick',[])
% xlabel('% Gait Cycle')
% title('Shifted EMG')
% axis([0 100 0 1])

EMGscales = [ones(optParams.nTrials_l,1)*emgScale(1:nMuscEmg); ones(optParams.nTrials_r,1)*emgScale(nMuscEmg+1:end);];
EMGscales = permute(EMGscales, [3 1 2]);
EMG = EMG.*EMGscales(ones(nptsInterp,1),:,:);

% subplot(1,5,3)
% hold on
% plot(0:100,EMG(21:121,50,24),'k','LineWidth',3)
% set(gca,'FontSize',24,'xtick',[0 100],'ytick',[])
% xlabel('% Gait Cycle')
% title('Scaled EMG')
% axis([0 100 0 1])

Excitations = EMG((nptsInterp-1)/7+1:interpStep:nptsInterp-(nptsInterp-1)/7,:,:);

b2(1,1,:) = 1./tDeact;
b1(1,1,:) = 1./tAct-b2;
b3 = b1(ones(nptsLong,1), ones(nTrials,1),:).*EMG + b2(ones(nptsLong,1), ones(nTrials,1),:);
dt(1,:,1) = mean(diff(Time));
b4 = 2*dt(ones(nptsLong,1),:,ones(nMuscEmg,1)).*b3; % NOTE (Nathan): I'm not sure exactly what this is, but it seems like it works

EMG = b4.*EMG;

a = zeros(size(EMG));
for j = 3:nptsLong
    a(j,:,:) = (EMG(j,:,:) + 4.*a(j-1,:,:) - a(j-2,:,:))./(b4(j,:,:)+3);
end

% % Test different activation calculation to see if it behaves the same
% 
% c2(1,1,:) = 1./tDeact;
% c1(1,1,:) = 1./tAct - c2;
% c1 = c1(ones(nptsLong,1),ones(nTrials,1),:);
% c2 = c2(ones(nptsLong,1),ones(nTrials,1),:);
% dt(1,:,1) = mean(diff(Time));
% dt = dt(ones(nptsLong,1),:,ones(nMuscEmg,1));
% 
% a = zeros(size(EMG));
% for jjj = 3:(nptsLong-1)
%     aDot_i = (EMG(jjj,:,:) - a(jjj,:,:)).*(c1(jjj,:,:).*EMG(jjj,:,:) + c2(jjj,:,:));
%     aDot_im1 = (EMG(jjj-1,:,:) - a(jjj-1,:,:)).*(c1(jjj-1,:,:).*EMG(jjj-1,:,:) + c2(jjj-1,:,:));
%     aDot_im2 = (EMG(jjj-2,:,:) - a(jjj-2,:,:)).*(c1(jjj-2,:,:).*EMG(jjj-2,:,:) + c2(jjj-2,:,:));
%     
%     a(jjj+1,:,:) = a(jjj,:,:) + dt(jjj,:,:).*(aDot_im2 + 4*aDot_im1 + aDot_i)/3;
% end

% a = zeros(nptsInterp,nTrials,nMuscEmg);
% AbsTol = 1e-4;
% RelTol = 1e-3;
% options = odeset('AbsTol',AbsTol,'RelTol',RelTol);
% for iii = 1:nTrials
%     a0 = 0;
%     t = [Time(1,iii),Time(end,iii)];
%     
%     for jjj = 1:nMuscEmg
%         emgSpline = spline(Time(:,iii),EMG(:,iii,jjj));
%         [tTemp,aTemp] = ode45(@heActivationDynamics,t,a0,options,emgSpline,tAct(jjj),tDeact(jjj));
%         a(:,iii,jjj) = spline(tTemp,aTemp,Time(:,iii));
%     end
% end


a = a((interpStep*(21-1)/SampleStep+1):interpStep:(interpStep*(121-1)/SampleStep+1),:,:);
% aRK = a;
% 
% close all
% plot(EMG(:,3,1),'k')
% hold on
% plot(aE(:,45,33),'b')
% hold on
% plot(aRK(:,45,33),'r')

% subplot(1,5,4)
% hold on
% Anonlin(24) = 0.15;
% plot(0:100,a(:,50,24),'k','LineWidth',3)
% set(gca,'FontSize',24,'xtick',[0 100],'ytick',[])
% xlabel('% Gait Cycle')
% title('Activation')
% axis([0 100 0 1])
% set(gca,'FontSize',24)

a = reshape(a, nptsShort*nTrials, nMuscEmg);
a(a<0) = 0;

% From optimization FindNonlinearityLogFunc.m
nonlincoefs = [29.280183270562596   4.107869238218326 1.000004740962477...
    -7.623282868703527 17.227022969058535   0.884220539986325];

a = (1-aNonlin(onesCol,:)).*a+(aNonlin(onesCol,:)).*...
    (nonlincoefs(4)./(nonlincoefs(1)*(a+nonlincoefs(6)).^nonlincoefs(5)+nonlincoefs(2))+nonlincoefs(3));

% a = reshape(a,nptsShort, nTrials, nMusc);
% subplot(1,5,5)
% hold on
% plot(0:100,a(:,50,24),'k','LineWidth',3)
% set(gca,'FontSize',24,'xtick',[0 100],'ytick',[])
% xlabel('% Gait Cycle')
% title('Nonlinear Activation')
% axis([0 100 0 1])
% set(gca,'FontSize',24)

end

function FMtilda = FLCurve(lMtilda)

b11 = 0.814483478343008;
b21 = 1.055033428970575;
b31 = 0.162384573599574;
b41 = 0.063303448465465;
b12 = 0.433004984392647;
b22 = 0.716775413397760;
b32 = -0.029947116970696;
b42 = 0.200356847296188;
b13 = 0.1;
b23 = 1;
b33 = 0.5*sqrt(0.5);
b43 = 0;

FMtilda = b11*exp(-0.5*(lMtilda-b21).^2./(b31+b41*lMtilda).^2)...
    +b12*exp(-0.5*(lMtilda-b22).^2./(b32+b42*lMtilda).^2)+...
    b13*exp(-0.5*(lMtilda-b23).^2./(b33+b43*lMtilda).^2);
end

function FMtilda = FVCurve(vMtilda)
% Coefficients determined manually

d1 = -8.665833008200368;
d2 = 7.623504943764548;
d3 = 3.565407127548163;
d4 = 0.968466940650055;
d5 = -0.394453556647392;
d6 = 8.169548497993912;

FMtilda = d1+d2*atan(d3+d4*atan(d5+d6*vMtilda));
end

function PFtilda = passiveForce(lMtilda)

c1 = 3.384513491001194;
c2 = -74.090638944399615;
c3 = -2.443789456896774;

PFtilda = c1*exp(c2*(exp(lMtilda)).^c3);
end

function [moments, lmtilda, vmtilda, HipMA, HipAAMA, KneeMA, AnkleMA,...
    SubtalarMA, muscMoments, passiveF, Lmt, MuscForce] = calcMoments(optParams, DoMomentCalcs)

% extract values from optParams structure
nMuscTotal = optParams.nMuscTotal;
nframesAll = optParams.nframesAll;
alpha = optParams.alpha;
onesCol = optParams.onesCol;
Mat = optParams.Mat;
MuscRef = optParams.MuscRef;
coefs = optParams.coefs;

% MACurvatureOffset = optParams.MACurvatureOffset;
lts = optParams.lts;
lmo = optParams.lmo;
% Fmax = optParams.Fmax;
a = optParams.a;

HipMA = zeros(nframesAll,nMuscTotal);
HipAAMA = zeros(nframesAll,nMuscTotal);
KneeMA = zeros(nframesAll,nMuscTotal);
AnkleMA = zeros(nframesAll,nMuscTotal);
SubtalarMA = zeros(nframesAll,nMuscTotal);
Lmt = zeros(nframesAll,nMuscTotal);
Vmt = zeros(nframesAll,nMuscTotal);

k = 1;
Ref1 = 1:nframesAll;
Ref2 = nframesAll+1:2*nframesAll;
Ref3 = 2*nframesAll+1:3*nframesAll;
Ref4 = 3*nframesAll+1:4*nframesAll;
Ref5 = 4*nframesAll+1:5*nframesAll;
for i = 1:nMuscTotal
    if MuscRef(i) == 1
        Vec = Mat{1}*coefs(k:k+18);
        Lmt(:,i) = Vec(Ref1);
        Vmt(:,i) = Vec(Ref2);
        HipMA(:,i) = Vec(Ref3);
        HipAAMA(:,i) = Vec(Ref4);
        k = k+19;
    elseif MuscRef(i) == 2
        Vec = Mat{2}*coefs(k:k+21);
        Lmt(:,i) = Vec(Ref1);
        Vmt(:,i) = Vec(Ref2);
        HipMA(:,i) = Vec(Ref3);
        HipAAMA(:,i) = Vec(Ref4);
        KneeMA(:,i) = Vec(Ref5);
        k = k+22;
    elseif MuscRef(i) == 3
        Vec = Mat{3}*coefs(k:k+3);
        Lmt(:,i) = Vec(Ref1);
        Vmt(:,i) = Vec(Ref2);
        KneeMA(:,i) = Vec(Ref3);
        k = k+4;
    elseif MuscRef(i) == 4
        Vec = Mat{4}*coefs(k:k+12);
        Lmt(:,i) = Vec(Ref1);
        Vmt(:,i) = Vec(Ref2);
        KneeMA(:,i) = Vec(Ref3);
        AnkleMA(:,i) = Vec(Ref4);
        SubtalarMA(:,i) = Vec(Ref5);
        k = k+13;
    elseif MuscRef(i) == 5
        Vec = Mat{5}*coefs(k:k+9);
        Lmt(:,i) = Vec(Ref1);
        Vmt(:,i) = Vec(Ref2);
        AnkleMA(:,i) = Vec(Ref3);
        SubtalarMA(:,i) = Vec(Ref4);
        k = k+10;
    end
end

lmtilda = (Lmt-onesCol*lts)./(onesCol*(lmo.*cos(alpha)));
vmtilda = Vmt./(optParams.VmaxFactor*onesCol*(lmo.*cos(alpha)));

Fmax = (optParams.Vmusc./lmo)*optParams.sigma;

if DoMomentCalcs
    passiveF = onesCol*(Fmax).*passiveForce(lmtilda);
    MuscForce = onesCol*(Fmax.*cos(alpha)).*a.*FLCurve(lmtilda).*FVCurve(vmtilda)+passiveF;
    
    muscMoments(:,:,1) = HipMA.*MuscForce;
    muscMoments(:,:,2) = HipAAMA.*MuscForce;
    muscMoments(:,:,3) = KneeMA.*MuscForce;
    muscMoments(:,:,4) = AnkleMA.*MuscForce;
    muscMoments(:,:,5) = SubtalarMA.*MuscForce;
    
    moments = permute(sum(muscMoments,2),[1 3 2]);
else
    passiveF = 0;
    moments = 0;
    muscMoments = 0;
end
end

function cost = costFuncMain(optParams)
% The main body of the cost function which is called from all other cost
% functions

% get needed values from optParams matrix
nMuscTotal = optParams.nMuscTotal;
nMuscEmg = optParams.nMuscEmg;
IDloads = optParams.IDloads;
nframesAll = optParams.nframesAll;
onesCol = optParams.onesCol;
tAct = optParams.tAct;
aNonlin = optParams.aNonlin;
ltsScale = optParams.ltsScale;
lmoScale = optParams.lmoScale;
emgScale = optParams.emgScale;
tDelay = optParams.tDelay;


% calculate model moments and other muscle parameters
[ModelMoments, lmtilda, ~, HipMA, HipAAMA, KneeMA, AnkleMA, SubtalarMA, ~, passiveF,Lmt] = calcMoments(optParams,1);

% calculate passive moments to match with thelen curves (activation is
% zero)
optParams.a = optParams.aThelen;
optParams.Mat = optParams.MatThelen;
optParams.onesCol = optParams.onesColThelen;
optParams.nframesAll = optParams.nframesAllThelen;

[PassiveM] = calcMoments(optParams,1);
% keyboard;
% remove joint that thelen has no data for (hip aa and subtalar)
PassiveM = PassiveM(optParams.PassiveMThelen(:,:)~=0);
PassiveMThelen = optParams.PassiveMThelen(optParams.PassiveMThelen~=0);

% calculate moment errors
costID = (ModelMoments-IDloads);%; LowBoundPenalty(:); UpperBoundPenalty(:)];
costID(:,1) = costID(:,1)*2; % For weighting the hip more heavily
% costID(:,2:5) = costID(:,2:5)/10;

costPassive = [(PassiveM-PassiveMThelen)];
% costPassive = 0;


% extract muscle pairing information
ActivationPairs = optParams.ActivationPairs;
lmtildaPairs = optParams.lmtildaPairs;
HipMAPairs = optParams.HipMAPairs;
HipAAMAPairs = optParams.HipAAMAPairs;
KneeMAPairs = optParams.KneeMAPairs;
AnkleMAPairs = optParams.AnkleMAPairs;
SubtalarMAPairs = optParams.SubtalarMAPairs;

% Costs for violating activation similarity constraints
ActivationPenalty = 0;
for i = 1:length(ActivationPairs)
    ActivationPenalty = ActivationPenalty+std(emgScale(ActivationPairs{i}))+std(emgScale(ActivationPairs{i}+nMuscEmg));
end

% Costs for violating time delay similarity constraints
tDelayPenalty = 0;
% for i = 1:length(ActivationPairs)
%     tDelayPenalty = tDelayPenalty+std(tDelay(ActivationPairs{i}))+std(tDelay(ActivationPairs{i}+nMusc));
% end

% Costs for violating lmtilda similarity constraints
lmtildaPenalty = 0;
for i = 1:length(lmtildaPairs)
    lmtildaPenalty = lmtildaPenalty+std(lmtilda(:,lmtildaPairs{i}),0,2)./((nframesAll)^0.5);
end

% Costs for violating moment arm similarity constraints
MAPenalty = 0;
for i = 1:length(HipMAPairs)
    MAPenalty = MAPenalty+std(HipMA(:,HipMAPairs{i}),[],2)./((nframesAll)^0.5);
end

for i = 1:length(HipAAMAPairs)
    MAPenalty = MAPenalty+std(HipAAMA(:,HipAAMAPairs{i}),[],2)./((nframesAll)^0.5);
end

for i = 1:length(KneeMAPairs)
    MAPenalty = MAPenalty+std(KneeMA(:,KneeMAPairs{i}),[],2)./((nframesAll)^0.5);
end

for i = 1:length(AnkleMAPairs)
    MAPenalty = MAPenalty+std(AnkleMA(:,AnkleMAPairs{i}),[],2)./((nframesAll)^0.5);
end

for i = 1:length(SubtalarMAPairs)
    MAPenalty = MAPenalty+std(SubtalarMA(:,SubtalarMAPairs{i}),[],2)./((nframesAll)^0.5);
end

% cost for having moment arms and muscle lengths change too much from
% initial values
LmtMean = mean(Lmt);
LmtMeanOrig = mean(optParams.Lmt);
HipMAMean = mean(HipMA);
HipMAMeanOrig = mean(optParams.HipMA);
HipAAMAMean = mean(HipAAMA);
HipAAMAMeanOrig = mean(optParams.HipAAMA);
KneeMAMean = mean(KneeMA);
KneeMAMeanOrig = mean(optParams.KneeMA);
AnkleMAMean = mean(AnkleMA);
AnkleMAMeanOrig = mean(optParams.AnkleMA);
SubtalarMAMean = mean(SubtalarMA);
SubtalarMAMeanOrig = mean(optParams.SubtalarMA);

% Calculate max absolute value of muscle geometry curves for normalization
LmtRange = range(optParams.Lmt);
HipMARange = range(optParams.HipMA);
HipAAMARange = range(optParams.HipAAMA);
KneeMARange = range(optParams.KneeMA);
AnkleMARange = range(optParams.AnkleMA);
SubtalarMARange = range(optParams.SubtalarMA);

% Penalize differences in mean values of moment arms
LmtPenMean = (LmtMean-LmtMeanOrig)./LmtRange;
HipMAPenMean = (HipMAMean-HipMAMeanOrig)./HipMARange;
HipAAMAPenMean = (HipAAMAMean-HipAAMAMeanOrig)./HipAAMARange;
KneeMAPenMean = (KneeMAMean-KneeMAMeanOrig)./KneeMARange;
AnkleMAPenMean = (AnkleMAMean-AnkleMAMeanOrig)./AnkleMARange;
SubtalarMAPenMean = (SubtalarMAMean-SubtalarMAMeanOrig)./SubtalarMARange;

% Penalize changes in shape of moment arms
LmtPen = (Lmt-optParams.Lmt+onesCol*(LmtMeanOrig-LmtMean))./(onesCol*LmtRange);
HipMAPen = (HipMA-optParams.HipMA+onesCol*(HipMAMeanOrig-HipMAMean))./(onesCol*range(optParams.HipMA));
HipAAMAPen = (HipAAMA-optParams.HipAAMA+onesCol*(HipAAMAMeanOrig-HipAAMAMean))./(onesCol*HipAAMARange);
KneeMAPen = (KneeMA-optParams.KneeMA+onesCol*(KneeMAMeanOrig-KneeMAMean))./(onesCol*KneeMARange);
AnkleMAPen = (AnkleMA-optParams.AnkleMA+onesCol*(AnkleMAMeanOrig-AnkleMAMean))./(onesCol*AnkleMARange);
SubtalarMAPen = (SubtalarMA-optParams.SubtalarMA+onesCol*(SubtalarMAMeanOrig-SubtalarMAMean))./(onesCol*SubtalarMARange);

% combine all costs into single vector
cost = [costID(:)/(sqrt(1)*(5*nframesAll)^0.5); ... % moment matching cost, time varying
    (tAct(:)-.015)/(.015*nMuscEmg.^0.5);... % activation value costs, time invariant
    ActivationPenalty(:);... % activation pair cost, time invariant
    10*tDelayPenalty(:);... % time delay pair cost, time invariant
    sqrt(10)*aNonlin(:)/(0.25*nMuscEmg^(0.5));... % anonlin cost, time invariant
    (ltsScale(:)-1)/(.25*nMuscTotal^(0.5));... % lts change cost, time invariant
    (lmoScale(:)-1)/(.25*nMuscTotal^(0.5)); ... % lmo change cost, time invariant
    LmtPenMean(:)/(1*(nMuscTotal).^0.5); ... % lmt mean change cost, time invariant
    HipMAPenMean(:)/(0.5*(nMuscTotal).^0.5); ... % Hip MA mean change cost, time invariant
    HipAAMAPenMean(:)/(0.5*(nMuscTotal).^0.5); ... % Hip AA MA mean change cost, time invariant
    KneeMAPenMean(:)/(0.5*(nMuscTotal).^0.5); ... % Knee MA mean change cost, time invariant
    AnkleMAPenMean(:)/(0.5*(nMuscTotal).^0.5); ... % Ankle MA mean change cost, time invariant
    SubtalarMAPenMean(:)/(0.5*(nMuscTotal).^0.5); ... % Subtalar MA mean change cost, time invariant
    LmtPen(:)/(0.25*(nframesAll*nMuscTotal).^0.5); ... % lmt change cost, time varying
    HipMAPen(:)/(0.25*(nframesAll*nMuscTotal).^0.5); ... % Hip MA change cost, time varying
    HipAAMAPen(:)/(0.25*(nframesAll*nMuscTotal).^0.5); ... % Hip AA MA change cost, time varying
    KneeMAPen(:)/(0.125*(nframesAll*nMuscTotal).^0.5); ... % Knee MA change cost, time varying
    AnkleMAPen(:)/(0.125*(nframesAll*nMuscTotal).^0.5); ... % Ankle MA change cost, time varying
    SubtalarMAPen(:)/(0.125*(nframesAll*nMuscTotal).^0.5); ... % Subtalar MA change cost, time varying
    MAPenalty(:)/.02; ...% moment arm similarity penalty costs, time varying
    sqrt(1e0)*costPassive(:)/(numel(PassiveM)^0.5); ... % minimize passive force, time varying NOTE (Nathan): passive cost weighting was at 1e0
    lmtildaPenalty(:);... % lmtilda similarity cost, time varying
    (emgScale(:)-1)/(0.5*(2*nMuscEmg).^0.5)]/sqrt(10); 

% cost = costID(:)/(sqrt(10)*(5*nframesAll)^0.5);

cost(isnan(cost))=0;

end

function cost = costFunc1(guess, optParams)

synergyCommands_l = optParams.synergyCommands_l;
synergyCommands_r = optParams.synergyCommands_r;

nMuscEmg = optParams.nMuscEmg;
nMuscReconst = optParams.nMuscReconst;
nReconstDup = optParams.nReconstDup;
nSyn = optParams.nSyn;

% extract values from optimizer guess
reconstSynWeights_l = reshape(guess(1,1:(nSyn*(nMuscReconst-nReconstDup))),[nSyn,(nMuscReconst-nReconstDup)]); %Unknown muscle order: Iliopsoas, TibPost, EDL, FDL
index = nSyn*(nMuscReconst-nReconstDup);
reconstSynWeights_r = reshape(guess(1,(index+1):(index+nSyn*(nMuscReconst-nReconstDup))),[nSyn,(nMuscReconst-nReconstDup)]); %Unknown muscle order: Iliopsoas, TibPost, EDL, FDL
index = index + nSyn*(nMuscReconst-nReconstDup);
tDelay = guess(1,1:2)/10;
index = index + 2;
tAct = guess(1,(index+1):(index+nMuscEmg))/100;
index = index + nMuscEmg;
tDeact = 4*tAct;
aNonlin = guess(1,(index+1):(index+nMuscEmg));
index = index + nMuscEmg;
emgScale = guess(1,(index+1):(index+2*nMuscEmg));
VmaxFactor = 10;
lmoScale = optParams.otherParams.lmoScale;
ltsScale = optParams.otherParams.ltsScale;

if optParams.GeometricAdjustments
    coefsOffset = optParams.otherParams.coefsOffset.*optParams.coefsScaleFactors; %optParams.otherParams(1,(1:optParams.ncoefsFit)+4*nMuscEmg).*optParams.coefsScaleFactors;
else
    coefsOffset = 0;
end

lts = optParams.lts.*ltsScale;
lmo = optParams.lmo.*lmoScale;

% write new parameter values to optimization parameters matrix
optParams.lts = lts;
optParams.lmo = lmo;
optParams.tDelay = tDelay;
optParams.tAct = tAct;
optParams.tDeact = tDeact;
optParams.aNonlin = aNonlin;
optParams.VmaxFactor = VmaxFactor;
optParams.emgScale = emgScale;
optParams.coefs = coefsOffset'+optParams.coefsOrig;
optParams.ltsScale = ltsScale;
optParams.lmoScale = lmoScale;

[a,~] = EMGtoAct(optParams);

unknownActs_l = synergyCommands_l*reconstSynWeights_l;
unknownActs_r = synergyCommands_r*reconstSynWeights_r;
unknownActs = [unknownActs_l;unknownActs_r];
% keyboard;
optParams.a = [a,unknownActs(:,1),unknownActs]; % NOTE: this part is specific to the muscles and EMG you have

cost = costFuncMain(optParams);
cost = sum(cost.^2);
end

function [c,ceq] = nonlcon1(guess, optParams)

synergyCommands_l = optParams.synergyCommands_l;
synergyCommands_r = optParams.synergyCommands_r;
nMuscReconst = optParams.nMuscReconst;
nReconstDup = optParams.nReconstDup;
nSyn = optParams.nSyn;

reconstSynWeights_l = reshape(guess(1,1:(nSyn*(nMuscReconst-nReconstDup))),[nSyn,(nMuscReconst-nReconstDup)]); %Unknown muscle order: Iliopsoas, TibPost, EDL, FDL
index = nSyn*(nMuscReconst-nReconstDup);
reconstSynWeights_r = reshape(guess(1,(index+1):(index+nSyn*(nMuscReconst-nReconstDup))),[nSyn,(nMuscReconst-nReconstDup)]); %Unknown muscle order: Iliopsoas, TibPost, EDL, FDL

ceq = [];

unknownActs_l = synergyCommands_l*reconstSynWeights_l;
unknownActs_r = synergyCommands_r*reconstSynWeights_r;
unknownActs = [unknownActs_l;unknownActs_r];


% assign values to c matrix
c = unknownActs - 1;

end

function cost = costFunc2(guess, optParams)
synergyCommands_l = optParams.synergyCommands_l;
synergyCommands_r = optParams.synergyCommands_r;
nMuscEmg = optParams.nMuscEmg;
nMuscTotal = optParams.nMuscTotal;
nSyn = optParams.nSyn;
nMuscReconst = optParams.nMuscReconst;
nReconstDup = optParams.nReconstDup;

% extract values from optimizer guess
synWeightsTemp = optParams.otherParams.reconstSynWeights(1,1:(nSyn*(nMuscReconst-nReconstDup)));
reconstSynWeights_l = reshape(synWeightsTemp,[nSyn,(nMuscReconst-nReconstDup)]);
synWeightsTemp = optParams.otherParams.reconstSynWeights(1,(nSyn*(nMuscReconst-nReconstDup)+1):(2*nSyn*(nMuscReconst-nReconstDup)));
reconstSynWeights_r = reshape(synWeightsTemp,[nSyn,(nMuscReconst-nReconstDup)]);
tDelay = optParams.otherParams.tDelay/10;
tAct = optParams.otherParams.tAct/100;
tDeact = 4*tAct;
aNonlin = optParams.otherParams.aNonlin;
VmaxFactor = 10;
emgScale = optParams.otherParams.emgScale;
lmoScale = guess(1:nMuscTotal);
index = nMuscTotal;
ltsScale = guess((index+1):(index+nMuscTotal));

if optParams.GeometricAdjustments
    coefsOffset = optParams.otherParams.coefsOffset.*optParams.coefsScaleFactors; %optParams.otherParams(1,(1:optParams.ncoefsFit)+4*nMuscEmg).*optParams.coefsScaleFactors;
else
    coefsOffset = 0;
end

lts = optParams.lts.*ltsScale;
lmo = optParams.lmo.*lmoScale;

% write new parameter values to optimization parameters matrix
optParams.lts = lts;
optParams.lmo = lmo;
optParams.tDelay = tDelay;
optParams.tAct = tAct;
optParams.tDeact = tDeact;
optParams.aNonlin = aNonlin;
optParams.VmaxFactor = VmaxFactor;
optParams.emgScale = emgScale;
optParams.coefs = coefsOffset'+optParams.coefsOrig;
optParams.ltsScale = ltsScale;
optParams.lmoScale = lmoScale;

[a,~] = EMGtoAct(optParams);
% synergyCommands = optParams.synergyCommands;
% sizeA = size(a);
% aReshape = zeros(sizeA(1)*sizeA(2),sizeA(3));
% for iii = 1:size(a,2)
%     aReshape(((iii-1)*sizeA(1)+1):(iii*sizeA(1)),:) = squeeze(a(:,iii,:));
% end
% if isempty(synergyCommands)
%     synergyCommands = optParams.synergyCommands;
%     synergyWeights = optParams.synergyWeights;
% end
% [synergyCommands,synergyWeights] = nnmf(a,optParams.nSyn,'w0',synergyCommands,'h0',synergyWeights);
unknownActs_l = synergyCommands_l*reconstSynWeights_l;
unknownActs_r = synergyCommands_r*reconstSynWeights_r;
unknownActs = [unknownActs_l;unknownActs_r];
optParams.a = [a,unknownActs(:,1),unknownActs]; % NOTE: this part is specific to the muscles and EMG you have

cost = costFuncMain(optParams);
cost = sum(cost.^2);
end

function [c,ceq] = nonlcon2(guess, optParams)

ceq = [];

nMuscTotal = optParams.nMuscTotal;
nframesAll = optParams.nframesAll;

% extract values from optimizer guess
VmaxFactor = 10;
lmoScale = guess(1,(1:nMuscTotal)+0*nMuscTotal);
ltsScale = guess(1,(1:nMuscTotal)+1*nMuscTotal);
if optParams.GeometricAdjustments
    coefsOffset = optParams.otherParams.coefsOffset.*optParams.coefsScaleFactors;
else
    coefsOffset = 0;
end

lts = optParams.lts.*ltsScale;
lmo = optParams.lmo.*lmoScale;

% write new parameter values to optimization parameters matrix
optParams.lts = lts;
optParams.lmo = lmo;
optParams.VmaxFactor = VmaxFactor;
optParams.coefs = coefsOffset'+optParams.coefsOrig;

% set activations to zero
a = zeros(nframesAll, nMuscTotal);

optParams.a = a;

% calculate lmtilda and vmtilda
[~, lmtilda, vmtilda] = calcMoments(optParams,0);

softmaxlmtildaCon = log(sum(exp(1000*(lmtilda-1.3))))/1000;
softminlmtildaCon = log(sum(exp(1000*(0.3-lmtilda))))/1000;
softminlmtildaCon2 = -log(sum(exp(1000*(1-lmtilda))))/1000;
softmaxlmtildaCon2 = -log(sum(exp(1000*(lmtilda-0.8))))/1000;
softmaxvmtilda = log(sum(exp(1000*vmtilda)))/1000;
softminvmtilda = -log(sum(exp(-1000*vmtilda)))/1000;

% assign values to c matrix
c = [softmaxlmtildaCon, softminlmtildaCon, softmaxvmtilda-1, softminvmtilda-1,...
   softmaxlmtildaCon2, softminlmtildaCon2];

c(c<-1e3) = -1;
c(c>1e3) = 1;

end

function cost = costFunc3(guess, optParams)
synergyCommands_l = optParams.synergyCommands_l;
synergyCommands_r = optParams.synergyCommands_r;
nSyn = optParams.nSyn;
nMuscReconst = optParams.nMuscReconst;
nReconstDup = optParams.nReconstDup;

% extract values from optimizer guess
synWeightsTemp = optParams.otherParams.reconstSynWeights(1,1:(nSyn*(nMuscReconst-nReconstDup)));
reconstSynWeights_l = reshape(synWeightsTemp,[nSyn,(nMuscReconst-nReconstDup)]);
synWeightsTemp = optParams.otherParams.reconstSynWeights(1,(nSyn*(nMuscReconst-nReconstDup)+1):(2*nSyn*(nMuscReconst-nReconstDup)));
reconstSynWeights_r = reshape(synWeightsTemp,[nSyn,(nMuscReconst-nReconstDup)]);
tDelay = optParams.otherParams.tDelay/10;
tAct = optParams.otherParams.tAct/100;
tDeact = 4*tAct;
aNonlin = optParams.otherParams.aNonlin;
VmaxFactor = 10;
emgScale = optParams.otherParams.emgScale;
lmoScale = optParams.otherParams.lmoScale;
ltsScale = optParams.otherParams.ltsScale;

if optParams.GeometricAdjustments
    coefsOffset = guess.*optParams.coefsScaleFactors;
else
    coefsOffset = 0;
end

lts = optParams.lts.*ltsScale;
lmo = optParams.lmo.*lmoScale;

% write new parameter values to optimization parameters matrix
optParams.lts = lts;
optParams.lmo = lmo;
optParams.tDelay = tDelay;
optParams.tAct = tAct;
optParams.tDeact = tDeact;
optParams.aNonlin = aNonlin;
optParams.VmaxFactor = VmaxFactor;
optParams.emgScale = emgScale;
optParams.coefs = coefsOffset'+optParams.coefsOrig;
optParams.ltsScale = ltsScale;
optParams.lmoScale = lmoScale;

[a,~] = EMGtoAct(optParams);
% synergyCommands = optParams.synergyCommands;
% sizeA = size(a);
% aReshape = zeros(sizeA(1)*sizeA(2),sizeA(3));
% for iii = 1:size(a,2)
%     aReshape(((iii-1)*sizeA(1)+1):(iii*sizeA(1)),:) = squeeze(a(:,iii,:));
% end
% if isempty(synergyCommands)
%     synergyCommands = optParams.synergyCommands;
%     synergyWeights = optParams.synergyWeights;
% end
% [synergyCommands,synergyWeights] = nnmf(a,optParams.nSyn,'w0',synergyCommands,'h0',synergyWeights);
unknownActs_l = synergyCommands_l*reconstSynWeights_l;
unknownActs_r = synergyCommands_r*reconstSynWeights_r;
unknownActs = [unknownActs_l;unknownActs_r];
optParams.a = [a,unknownActs(:,1),unknownActs]; % NOTE: this part is specific to the muscles and EMG you have

cost = costFuncMain(optParams);
cost = sum(cost.^2);
end

function [c,ceq] = nonlcon3(guess, optParams)

ceq = [];

nMuscTotal = optParams.nMuscTotal;
nframesAll = optParams.nframesAll;

% extract values from optimizer guess
VmaxFactor = 10;
lmoScale = optParams.otherParams.lmoScale;
ltsScale = optParams.otherParams.ltsScale;
coefsOffset = guess.*optParams.coefsScaleFactors;

lts = optParams.lts.*ltsScale;
lmo = optParams.lmo.*lmoScale;

% extract original moment arms curves
HipMA = optParams.HipMA;
HipAAMA = optParams.HipAAMA;
KneeMA = optParams.KneeMA;
AnkleMA = optParams.AnkleMA;
SubtalarMA = optParams.SubtalarMA;

% write new parameter values to optimization parameters matrix
optParams.lts = lts;
optParams.lmo = lmo;
optParams.VmaxFactor = VmaxFactor;
optParams.coefs = coefsOffset'+optParams.coefsOrig;

% set activations to zero, since they are unnecessary for constraints
a = zeros(nframesAll, nMuscTotal);

optParams.a = a;

% calculate muscle parameter values to be constrained
[~, lmtilda, vmtilda, HipMAnew, HipAAMAnew, KneeMAnew, AnkleMAnew, SubtalarMAnew,~,~,Lmtnew] = calcMoments(optParams,0);

% Remove sign constraints on muscles that can act in two directions at a
% joint (mainly occurs at the hip)
HipMultiDirectional = max(sign(HipMA))+min(sign(HipMA));
HipAAMultiDirectional = max(sign(HipAAMA))+min(sign(HipAAMA));
KneeMultiDirectional = max(sign(KneeMA))+min(sign(KneeMA));
AnkleMultiDirectional = max(sign(AnkleMA))+min(sign(AnkleMA));
SubtalarMultiDirectional = max(sign(SubtalarMA))+min(sign(SubtalarMA));

HipMA(:,HipMultiDirectional==0) = 0;
HipAAMA(:,HipAAMultiDirectional==0) = 0;
KneeMA(:,KneeMultiDirectional==0) = 0;
AnkleMA(:,AnkleMultiDirectional==0) = 0;
SubtalarMA(:,SubtalarMultiDirectional==0) = 0;

% calculate constraint that prevents moment arms from switching signs
HipMACon = optParams.onesCol(:)*mode(-sign(HipMA)).*HipMAnew(:,:);
HipAAMACon = optParams.onesCol(:)*mode(-sign(HipAAMA)).*HipAAMAnew(:,:);
KneeMACon = optParams.onesCol(:)*mode(-sign(KneeMA)).*KneeMAnew(:,:);
AnkleMACon = optParams.onesCol(:)*mode(-sign(AnkleMA)).*AnkleMAnew(:,:);
SubtalarMACon = optParams.onesCol(:)*mode(-sign(SubtalarMA)).*SubtalarMAnew(:,:);

HipMACon(:,sum(HipMACon)==0) = [];
HipAAMACon(:,sum(HipAAMACon)==0) = [];
KneeMACon(:,sum(KneeMACon)==0) = [];
AnkleMACon(:,sum(AnkleMACon)==0) = [];
SubtalarMACon(:,sum(SubtalarMACon)==0) = [];


softmaxlmtildaCon = log(sum(exp(1000*(lmtilda-1.3))))/1000;
softminlmtildaCon = log(sum(exp(1000*(0.3-lmtilda))))/1000;
softminlmtildaCon2 = -log(sum(exp(1000*(1-lmtilda))))/1000;
softmaxlmtildaCon2 = -log(sum(exp(1000*(lmtilda-0.8))))/1000;
softmaxvmtilda = log(sum(exp(1000*vmtilda)))/1000;
softminvmtilda = -log(sum(exp(-1000*vmtilda)))/1000;
softmaxHipMACon = log(sum(exp(10000*HipMACon)))/10000;
softmaxHipAAMACon = log(sum(exp(10000*HipAAMACon)))/10000;
softmaxKneeMACon = log(sum(exp(10000*KneeMACon)))/10000;
softmaxAnkleMACon = log(sum(exp(10000*AnkleMACon)))/10000;
softmaxSubtalarMACon = log(sum(exp(10000*SubtalarMACon)))/10000;

% assign values to c matrix
c = [softmaxlmtildaCon, softminlmtildaCon, softmaxvmtilda-1, softminvmtilda-1,...
    softmaxHipMACon, softmaxHipAAMACon, softmaxKneeMACon, softmaxAnkleMACon, softmaxSubtalarMACon,...
    softmaxlmtildaCon2, softminlmtildaCon2];

c(c<-1e3) = -1;
c(c>1e3) = 1;

end

function cost = costFunc4(guess, optParams)

synergyCommands_l = optParams.synergyCommands_l;
synergyCommands_r = optParams.synergyCommands_r;
nMuscReconst = optParams.nMuscReconst;
nReconstDup = optParams.nReconstDup;
nSyn = optParams.nSyn;

% extract values from optimizer guess
reconstSynWeights_l = reshape(guess(1,1:(nSyn*(nMuscReconst-nReconstDup))),[nSyn,(nMuscReconst-nReconstDup)]); %Unknown muscle order: Iliopsoas, TibPost, EDL, FDL
index = nSyn*(nMuscReconst-nReconstDup);
reconstSynWeights_r = reshape(guess(1,(index+1):(index+nSyn*(nMuscReconst-nReconstDup))),[nSyn,(nMuscReconst-nReconstDup)]); %Unknown muscle order: Iliopsoas, TibPost, EDL, FDL

tDelay = optParams.otherParams.tDelay/10;
tAct = optParams.otherParams.tAct/100;
tDeact = 4*tAct;
aNonlin = optParams.otherParams.aNonlin;
VmaxFactor = 10;
lmoScale = optParams.otherParams.lmoScale;
ltsScale = optParams.otherParams.ltsScale;
emgScale = optParams.otherParams.emgScale;

if optParams.GeometricAdjustments
    coefsOffset = optParams.otherParams.coefsOffset.*optParams.coefsScaleFactors; %optParams.otherParams(1,(1:optParams.ncoefsFit)+4*nMuscEmg).*optParams.coefsScaleFactors;
else
    coefsOffset = 0;
end

lts = optParams.lts.*ltsScale;
lmo = optParams.lmo.*lmoScale;

% write new parameter values to optimization parameters matrix
optParams.lts = lts;
optParams.lmo = lmo;
optParams.tDelay = tDelay;
optParams.tAct = tAct;
optParams.tDeact = tDeact;
optParams.aNonlin = aNonlin;
optParams.VmaxFactor = VmaxFactor;
optParams.emgScale = emgScale;
optParams.coefs = coefsOffset'+optParams.coefsOrig;
optParams.ltsScale = ltsScale;
optParams.lmoScale = lmoScale;

[a,~] = EMGtoAct(optParams);

unknownActs_l = synergyCommands_l*reconstSynWeights_l;
unknownActs_r = synergyCommands_r*reconstSynWeights_r;
unknownActs = [unknownActs_l;unknownActs_r];
% keyboard;
optParams.a = [a,unknownActs(:,1),unknownActs]; % NOTE: this part is specific to the muscles and EMG you have

cost = costFuncMain(optParams);
cost = sum(cost.^2);
end

function cost = costFunc(guess, optParams)

% persistent synergyCommands synergyWeights
% persistent synergyCommands_l synergyWeights_l  synergyCommands_r synergyWeights_r
synergyCommands_l = optParams.synergyCommands_l;
synergyCommands_r = optParams.synergyCommands_r;

nMuscTotal = optParams.nMuscTotal;
nMuscEmg = optParams.nMuscEmg;
nMuscReconst = optParams.nMuscReconst;
nReconstDup = optParams.nReconstDup;
nSyn = optParams.nSyn;
% nTrials_l = optParams.nTrials_l;
% nTrials_r = optParams.nTrials_r;
% nptsShort = optParams.nptsShort;

% extract values from optimizer guess
% reconstSynWeights_l = reshape(guess(1,1:(nSyn*(nMuscReconst-nReconstDup))),[nSyn,(nMuscReconst-nReconstDup)]); %Unknown muscle order: Iliopsoas, TibPost, EDL, FDL
% index = nSyn*(nMuscReconst-nReconstDup);
% reconstSynWeights_r = reshape(guess(1,(index+1):(index+nSyn*(nMuscReconst-nReconstDup))),[nSyn,(nMuscReconst-nReconstDup)]); %Unknown muscle order: Iliopsoas, TibPost, EDL, FDL
% index = index + nSyn*(nMuscReconst-nReconstDup);
synWeightsTemp = optParams.otherParams.reconstSynWeights(1,1:(nSyn*(nMuscReconst-nReconstDup)));
reconstSynWeights_l = reshape(synWeightsTemp,[nSyn,(nMuscReconst-nReconstDup)]);
synWeightsTemp = optParams.otherParams.reconstSynWeights(1,(nSyn*(nMuscReconst-nReconstDup)+1):(2*nSyn*(nMuscReconst-nReconstDup)));
reconstSynWeights_r = reshape(synWeightsTemp,[nSyn,(nMuscReconst-nReconstDup)]);
index = 0;
tDelay = guess(1,(index+1):(index+2))/10;
index = index + 2;
tAct = guess(1,(index+1):(index+nMuscEmg))/100;
index = index + nMuscEmg;
tDeact = 4*tAct;
aNonlin = guess(1,(index+1):(index+nMuscEmg));
index = index + nMuscEmg;
VmaxFactor = 10;
emgScale = guess(1,(index+1):(index+2*nMuscEmg));
index = index + 2*nMuscEmg;
lmoScale = guess(1,(index+1):(index+nMuscTotal));
index = index + nMuscTotal;
ltsScale = guess(1,(index+1):(index+nMuscTotal));
index = index + nMuscTotal;

if optParams.GeometricAdjustments
    coefsOffset = guess(1,(index+1):end).*optParams.coefsScaleFactors;
else
    coefsOffset = 0;
end

lts = optParams.lts.*ltsScale;
lmo = optParams.lmo.*lmoScale;

% write new parameter values to optimization parameters matrix
optParams.lts = lts;
optParams.lmo = lmo;
optParams.tDelay = tDelay;
optParams.tAct = tAct;
optParams.tDeact = tDeact;
optParams.aNonlin = aNonlin;
optParams.VmaxFactor = VmaxFactor;
optParams.emgScale = emgScale;
optParams.coefs = coefsOffset'+optParams.coefsOrig;
optParams.ltsScale = ltsScale;
optParams.lmoScale = lmoScale;

[a,~] = EMGtoAct(optParams);
% if isempty(synergyCommands)
%     synergyCommands = optParams.synergyCommands;
%     synergyWeights = optParams.synergyWeights;
% end
% sizeA = size(a);
% aReshape = zeros(sizeA(1)*sizeA(2),sizeA(3));
% for iii = 1:size(a,2)
%     aReshape(((iii-1)*sizeA(1)+1):(iii*sizeA(1)),:) = squeeze(a(:,iii,:));
% end
% if isempty(synergyCommands)
%     synergyCommands = optParams.synergyCommands;
%     synergyWeights = optParams.synergyWeights;
% end
% a_l = a(1:(nTrials_l*nptsShort),:);
% a_r = a((nTrials_l*nptsShort + 1):end,:);
% if isempty(synergyCommands_l)
%     synergyCommands_l = optParams.synergyCommands_l;
%     synergyWeights_l = optParams.synergyWeights_l;
%     synergyCommands_r = optParams.synergyCommands_r;
%     synergyWeights_r = optParams.synergyWeights_r;
% end
% [synergyCommands_l,synergyWeights_l] = nnmf(a_l,optParams.nSyn,'w0',synergyCommands_l,'h0',synergyWeights_l);
% unknownActs_l = synergyCommands_l*reconstSynWeights_l;
% [synergyCommands_r,synergyWeights_r] = nnmf(a_r,optParams.nSyn,'w0',synergyCommands_r,'h0',synergyWeights_r);
% unknownActs_r = synergyCommands_r*reconstSynWeights_r;
% unknownActs = [unknownActs_l;unknownActs_r];
unknownActs_l = synergyCommands_l*reconstSynWeights_l;
unknownActs_r = synergyCommands_r*reconstSynWeights_r;
unknownActs = [unknownActs_l;unknownActs_r];
optParams.a = [a,unknownActs(:,1),unknownActs]; % NOTE: this part is specific to the muscles and EMG you have

cost = costFuncMain(optParams);
cost = sum(cost.^2);
end

function [c,ceq] = nonlcon(guess, optParams)

% no equality constraints
ceq = [];

nMuscTotal = optParams.nMuscTotal;
nMuscEmg = optParams.nMuscEmg;
% nMuscReconst = optParams.nMuscReconst;
% nReconstDup = optParams.nReconstDup;
% nSyn = optParams.nSyn;
nframesAll = optParams.nframesAll;

% extract values from parameters matrix
VmaxFactor = 10;
% index = 2*nSyn*(nMuscReconst-nReconstDup) + 2 + 2*nMuscEmg + 2*nMuscEmg;
index = 2 + 2*nMuscEmg + 2*nMuscEmg;
lmoScale = guess(1,(index+1):(index+nMuscTotal));
index = index + nMuscTotal;
ltsScale = guess(1,(index+1):(index+nMuscTotal));
index = index + nMuscTotal;
if optParams.GeometricAdjustments
    coefsOffset = guess(1,(index+1):end).*optParams.coefsScaleFactors;
else
    coefsOffset = 0;
end

lmo = optParams.lmo.*lmoScale;
lts = optParams.lts.*ltsScale;

% extract original moment arms curves
HipMA = optParams.HipMA;
HipAAMA = optParams.HipAAMA;
KneeMA = optParams.KneeMA;
AnkleMA = optParams.AnkleMA;
SubtalarMA = optParams.SubtalarMA;

% write new parameter values to optimization parameters matrix
optParams.lts = lts;
optParams.lmo = lmo;
optParams.VmaxFactor = VmaxFactor;
optParams.coefs = coefsOffset'+optParams.coefsOrig;

% set activations to zero, since they are unnecessary for constraints
a = zeros(nframesAll, nMuscTotal);

optParams.a = a;

% calculate muscle parameter values to be constrained
[~, lmtilda, vmtilda, HipMAnew, HipAAMAnew, KneeMAnew, AnkleMAnew, SubtalarMAnew,~,~,Lmtnew] = calcMoments(optParams,0);

% Remove sign constraints on muscles that can act in two directions at a
% joint (mainly occurs at the hip)
HipMultiDirectional = max(sign(HipMA))+min(sign(HipMA));
HipAAMultiDirectional = max(sign(HipAAMA))+min(sign(HipAAMA));
KneeMultiDirectional = max(sign(KneeMA))+min(sign(KneeMA));
AnkleMultiDirectional = max(sign(AnkleMA))+min(sign(AnkleMA));
SubtalarMultiDirectional = max(sign(SubtalarMA))+min(sign(SubtalarMA));

HipMA(:,HipMultiDirectional==0) = 0;
HipAAMA(:,HipAAMultiDirectional==0) = 0;
KneeMA(:,KneeMultiDirectional==0) = 0;
AnkleMA(:,AnkleMultiDirectional==0) = 0;
SubtalarMA(:,SubtalarMultiDirectional==0) = 0;

% calculate constraint that prevents moment arms from switching signs
HipMACon = optParams.onesCol(:)*mode(-sign(HipMA)).*HipMAnew(:,:);
HipAAMACon = optParams.onesCol(:)*mode(-sign(HipAAMA)).*HipAAMAnew(:,:);
KneeMACon = optParams.onesCol(:)*mode(-sign(KneeMA)).*KneeMAnew(:,:);
AnkleMACon = optParams.onesCol(:)*mode(-sign(AnkleMA)).*AnkleMAnew(:,:);
SubtalarMACon = optParams.onesCol(:)*mode(-sign(SubtalarMA)).*SubtalarMAnew(:,:);

HipMACon(:,sum(HipMACon)==0) = [];
HipAAMACon(:,sum(HipAAMACon)==0) = [];
KneeMACon(:,sum(KneeMACon)==0) = [];
AnkleMACon(:,sum(AnkleMACon)==0) = [];
SubtalarMACon(:,sum(SubtalarMACon)==0) = [];


softmaxlmtildaCon = log(sum(exp(1000*(lmtilda-1.3))))/1000;
softminlmtildaCon = log(sum(exp(1000*(0.3-lmtilda))))/1000;
softminlmtildaCon2 = -log(sum(exp(1000*(1-lmtilda))))/1000;
softmaxlmtildaCon2 = -log(sum(exp(1000*(lmtilda-0.8))))/1000;
softmaxvmtilda = log(sum(exp(1000*vmtilda)))/1000;
softminvmtilda = -log(sum(exp(-1000*vmtilda)))/1000;
softmaxHipMACon = log(sum(exp(10000*HipMACon)))/10000;
softmaxHipAAMACon = log(sum(exp(10000*HipAAMACon)))/10000;
softmaxKneeMACon = log(sum(exp(10000*KneeMACon)))/10000;
softmaxAnkleMACon = log(sum(exp(10000*AnkleMACon)))/10000;
softmaxSubtalarMACon = log(sum(exp(10000*SubtalarMACon)))/10000;


% assign values to c matrix
c = [softmaxlmtildaCon, softminlmtildaCon, softmaxvmtilda-1, softminvmtilda-1,...
    softmaxHipMACon, softmaxHipAAMACon, softmaxKneeMACon, softmaxAnkleMACon, softmaxSubtalarMACon,...
    softmaxlmtildaCon2, softminlmtildaCon2];

c(c<-1e3) = -1;
c(c>1e3) = 1;

end

% Plotting Functions
function makeParameterChangePlots(optParams)

tAct = optParams.tAct;
aNonlin = optParams.aNonlin;
ltsScale = optParams.ltsScale;
lmoScale = optParams.lmoScale;
emgScale = optParams.emgScale;
nMuscTotal = optParams.nMuscTotal;
nMuscEmg = optParams.nMuscEmg;

% plot design variables
muscLabels = optParams.MuscLabels;
muscLabelsEmg = optParams.muscLabelsEmg;
fig = figure('units','normalized','outerposition',[0 0 1 1]);
bar(tAct*1000)
title('Muscle Activation Time Constants')
ylabel('Time (ms)')
set(gca, 'XTick', 1:nMuscEmg, 'XTickLabel', muscLabelsEmg, 'FontSize', 10);

fig = figure('units','normalized','outerposition',[0 0 1 1]);
bar(aNonlin)
title('Muscle Nonlinearity Parameter')
ylabel('Nonlinearity Param')
set(gca, 'XTick', 1:nMuscEmg, 'XTickLabel', muscLabelsEmg, 'FontSize', 10);

fig = figure('units','normalized','outerposition',[0 0 1 1]);
bar(ltsScale)
title('Muscle Lts Scale Factors')
ylabel('Scale Factor')
set(gca, 'XTick', 1:nMuscTotal, 'XTickLabel', muscLabels, 'FontSize', 10);

fig = figure('units','normalized','outerposition',[0 0 1 1]);
bar(lmoScale)
title('Muscle Lmo Scale Factors')
ylabel('Scale Factor')
set(gca, 'XTick', 1:nMuscTotal, 'XTickLabel', muscLabels, 'FontSize', 10);

fig = figure('units','normalized','outerposition',[0 0 1 1]);
bar(emgScale)
title('Muscle Activation Scale Factors')
ylabel('Scale Factor')
set(gca, 'XTick', 1:nMuscEmg, 'XTickLabel', [muscLabelsEmg muscLabelsEmg], 'FontSize', 10);

end

function makeIndividualMuscMomentPlots(moments,muscMoments,optParams)
muscRef = optParams.MuscRef;
hipFeMusc = (muscRef == 1)|(muscRef == 2);
hipAaMusc = (muscRef == 1)|(muscRef == 2);
kneeFeMusc = (muscRef == 2)|(muscRef == 3)|(muscRef == 4);
ankleFeMusc = (muscRef == 4)|(muscRef == 5);
subtalarMusc = (muscRef == 4)|(muscRef == 5);


moments = reshape(moments,optParams.nptsShort*optParams.nTrials,5);
figure
plot(-moments(:,1),'r', 'LineWidth',2)
hold on
plot(-optParams.IDloads(:,1),'b', 'LineWidth',2)
plot(-muscMoments(:,hipFeMusc,1))
title('Hip FE Moment Calibration Results')
xlabel('Time Frames')
ylabel('Moment (N-m)')
legend(['Hip FE Moment Muscles' 'Hip FE Moment ID' optParams.MuscLabels(hipFeMusc)])

figure
plot(-moments(:,2),'r', 'LineWidth',2)
hold on
plot(-optParams.IDloads(:,2),'b', 'LineWidth',2)
plot(-muscMoments(:,hipAaMusc,2))
title('Hip AA Moment Calibration Results')
xlabel('Time Frames')
ylabel('Moment (N-m)')
legend(['Hip AA Moment Muscles' 'Hip AA Moment ID' optParams.MuscLabels(hipAaMusc)])

figure
plot(-moments(:,3),'r', 'LineWidth',2)
hold on
plot(-optParams.IDloads(:,3),'b', 'LineWidth',2)
plot(-muscMoments(:,kneeFeMusc,3))
title('Knee FE Moment Calibration Results')
xlabel('Time Frames')
ylabel('Moment (N-m)')
legend(['Knee FE Moment Muscles' 'Knee FE Moment ID' optParams.MuscLabels(kneeFeMusc)])

figure
plot(-moments(:,4),'r', 'LineWidth',2)
hold on
plot(-optParams.IDloads(:,4),'b', 'LineWidth',2)
plot(-muscMoments(:,ankleFeMusc,4))
title('Ankle Moment Calibration Results')
xlabel('Time Frames')
ylabel('Moment (N-m)')
legend(['Ankle Moment Muscles' 'Ankle Moment ID' optParams.MuscLabels(ankleFeMusc)])

figure
plot(-moments(:,5),'r', 'LineWidth',2)
hold on
plot(-optParams.IDloads(:,5),'b', 'LineWidth',2)
plot(-muscMoments(:,subtalarMusc,5))
title('Subtalar Moment Calibration Results')
xlabel('Time Frames')
ylabel('Moment (N-m)')
legend(['Subtalar Moment Muscles' 'Subtalar Moment ID' optParams.MuscLabels(subtalarMusc)])
end

function makeMomentMatchingPlots(moments,IDloads,optParams)

nTrials_l = optParams.nTrials_l;
nTrials_r = optParams.nTrials_r;
% nTrialsToUse = optParams.nTrialsToUse;
PassiveMoments = optParams.PassiveMGait;


momentsMean_l = permute(mean(moments(:,1:nTrials_l,:),2),[1 3 2]);
momentsMin_l = permute(max(moments(:,1:nTrials_l,:),[],2),[1 3 2]);
momentsMax_l = permute(min(moments(:,1:nTrials_l,:),[],2),[1 3 2]);
PassiveMomentsMean_l = permute(min(PassiveMoments(:,1:nTrials_l,:),[],2),[1 3 2]);

IDloadsMean_l = permute(mean(IDloads(:,1:nTrials_l,:),2),[1 3 2]);
IDloadsMin_l = permute(max(IDloads(:,1:nTrials_l,:),[],2),[1 3 2]);
IDloadsMax_l = permute(min(IDloads(:,1:nTrials_l,:),[],2),[1 3 2]);

momentsMean_r = permute(mean(moments(:,(nTrials_l+1):(nTrials_l+nTrials_r),:),2),[1 3 2]);
momentsMin_r = permute(max(moments(:,(nTrials_l+1):(nTrials_l+nTrials_r),:),[],2),[1 3 2]);
momentsMax_r = permute(min(moments(:,(nTrials_l+1):(nTrials_l+nTrials_r),:),[],2),[1 3 2]);
PassiveMomentsMean_r = permute(min(PassiveMoments(:,(nTrials_l+1):(nTrials_l+nTrials_r),:),[],2),[1 3 2]);

IDloadsMean_r = permute(mean(IDloads(:,(nTrials_l+1):(nTrials_l+nTrials_r),:),2),[1 3 2]);
IDloadsMin_r = permute(max(IDloads(:,(nTrials_l+1):(nTrials_l+nTrials_r),:),[],2),[1 3 2]);
IDloadsMax_r = permute(min(IDloads(:,(nTrials_l+1):(nTrials_l+nTrials_r),:),[],2),[1 3 2]);

fig = figure('units','normalized','outerposition',[0 0 1 1]);
% figure;
subplot(2,5,1), plot(-PassiveMomentsMean_l(:,1),'r--', 'LineWidth',2)
% subplot(2,5,1), plot(squeeze(-IDloads(:,1:20,1)),'g')
% hold on
% subplot(2,5,1), plot(squeeze(-IDloads(:,21:nTrials_l,1)),'b')
subplot(2,5,1), plot(-momentsMean_l(:,1),'r', 'LineWidth',2)
hold on
subplot(2,5,1), plot(-IDloadsMean_l(:,1),'k', 'LineWidth',2)
% subplot(5,2,1);
% plot(-momentsMean_l(:,1),'r', 'LineWidth',2)
% hold on
% subplot(5,2,1);
% plot(-IDloadsMean_l(:,1),'k', 'LineWidth',2)

title('Hip FE Moment Calibration Results')
xlabel('Time Frames')
ylabel('Moment (N-m)')
axis([0 100 -50 100])
set(gca, 'FontSize', 14)

% figure;
subplot(2,5,2), plot(-PassiveMomentsMean_l(:,2),'r--', 'LineWidth',2)
% subplot(2,5,2), plot(squeeze(-IDloads(:,1:20,2)),'g')
% hold on
% subplot(2,5,2), plot(squeeze(-IDloads(:,10:nTrials_l,2)),'b')
subplot(2,5,2), plot(-momentsMean_l(:,2),'r', 'LineWidth',2)
hold on
subplot(2,5,2), plot(-IDloadsMean_l(:,2),'k', 'LineWidth',2)
% subplot(5,2,3);
% plot(-momentsMean_l(:,2),'r', 'LineWidth',2)
% hold on
% subplot(5,2,3);
% plot(-IDloadsMean_l(:,2),'k', 'LineWidth',2)

title('Hip AA Moment Calibration Results')
xlabel('Time Frames')
ylabel('Moment (N-m)')
axis([0 100 -50 100])
set(gca, 'FontSize', 14)

% figure;
subplot(2,5,3), plot(-PassiveMomentsMean_l(:,3),'r--', 'LineWidth',2)
% subplot(2,5,3), plot(squeeze(-IDloads(:,1:20,3)),'g')
% hold on
% subplot(2,5,3), plot(squeeze(-IDloads(:,10:nTrials_l,3)),'b')
subplot(2,5,3), plot(-momentsMean_l(:,3),'r', 'LineWidth',2)
hold on
subplot(2,5,3), plot(-IDloadsMean_l(:,3),'k', 'LineWidth',2)
% subplot(5,2,5);
% plot(-momentsMean_l(:,3),'r', 'LineWidth',2)
% hold on
% % subplot(5,2,5);
% plot(-IDloadsMean_l(:,3),'k', 'LineWidth',2)

title('Knee FE Moment Calibration Results')
xlabel('Time Frames')
ylabel('Moment (N-m)')
axis([0 100 -50 100])
set(gca, 'FontSize', 14)

% figure;
subplot(2,5,4), plot(-PassiveMomentsMean_l(:,4),'r--', 'LineWidth',2)
% subplot(2,5,4), plot(squeeze(-IDloads(:,1:20,4)),'g')
% hold on
% subplot(2,5,4), plot(squeeze(-IDloads(:,10:nTrials_l,4)),'b')
subplot(2,5,4), plot(-momentsMean_l(:,4),'r', 'LineWidth',2)
hold on
subplot(2,5,4), plot(-IDloadsMean_l(:,4),'k', 'LineWidth',2)
% % subplot(5,2,7);
% plot(-momentsMean_l(:,4),'r', 'LineWidth',2)
% hold on
% % subplot(5,2,7);
% plot(-IDloadsMean_l(:,4),'k', 'LineWidth',2)

title('Ankle FE Moment Calibration Results')
xlabel('Time Frames')
ylabel('Moment (N-m)')
axis([0 100 -50 100])
set(gca, 'FontSize', 14)

% figure;
subplot(2,5,5), plot(-PassiveMomentsMean_l(:,5),'r--', 'LineWidth',2)
% subplot(2,5,5), plot(squeeze(-IDloads(:,1:20,5)),'g')
% hold on
% subplot(2,5,5), plot(squeeze(-IDloads(:,10:nTrials_l,5)),'b')
subplot(2,5,5), plot(-momentsMean_l(:,5),'r', 'LineWidth',2)
hold on
subplot(2,5,5), plot(-IDloadsMean_l(:,5),'k', 'LineWidth',2)
% % subplot(5,2,9);
% plot(-momentsMean_l(:,5),'r', 'LineWidth',2)
% hold on
% % subplot(5,2,9);
% plot(-IDloadsMean_l(:,5),'k', 'LineWidth',2)

title('Subtalar IE Moment Calibration Results')
xlabel('Time Frames')
ylabel('Moment (N-m)')
axis([0 100 -50 100])
legend({'Average Model Moment' 'Average ID Moment'});% 'Model Min/Max Moment' 'ID Min/Max Moment'})
set(gca, 'FontSize', 14)

% figure;
subplot(2,5,6), plot(-PassiveMomentsMean_r(:,1),'r--', 'LineWidth',2)
% subplot(2,5,6), plot(squeeze(-IDloads(:,(nTrials_l+1):(nTrials_l+17),1)),'g')
% hold on
% subplot(2,5,6), plot(squeeze(-IDloads(:,(nTrials_l+18):end,1)),'b')
subplot(2,5,6), plot(-momentsMean_r(:,1),'r', 'LineWidth',2)
hold on
subplot(2,5,6), plot(-IDloadsMean_r(:,1),'k', 'LineWidth',2)
% % subplot(5,2,2);
% plot(-momentsMean_r(:,1),'r', 'LineWidth',2)
% hold on
% % subplot(5,2,2);
% plot(-IDloadsMean_r(:,1),'k', 'LineWidth',2)

title('Hip FE Moment Calibration Results')
xlabel('Time Frames')
ylabel('Moment (N-m)')
axis([0 100 -50 100])
% legend({'Average Model Moment' 'Average ID Moment'});% 'Model Min/Max Moment' 'ID Min/Max Moment'})
set(gca, 'FontSize', 14)

% figure;
subplot(2,5,7), plot(-PassiveMomentsMean_r(:,2),'r--', 'LineWidth',2)
% subplot(2,5,7), plot(squeeze(-IDloads(:,(nTrials_l+1):(nTrials_l+17),2)),'g')
% hold on
% subplot(2,5,7), plot(squeeze(-IDloads(:,(nTrials_l+18):end,2)),'b')
subplot(2,5,7), plot(-momentsMean_r(:,2),'r', 'LineWidth',2)
hold on
subplot(2,5,7), plot(-IDloadsMean_r(:,2),'k', 'LineWidth',2)
% % subplot(5,2,4);
% plot(-momentsMean_r(:,2),'r', 'LineWidth',2)
% hold on
% % subplot(5,2,4);
% plot(-IDloadsMean_r(:,2),'k', 'LineWidth',2)

title('Hip AA Moment Calibration Results')
xlabel('Time Frames')
ylabel('Moment (N-m)')
axis([0 100 -50 100])
set(gca, 'FontSize', 14)

% figure;
subplot(2,5,8), plot(-PassiveMomentsMean_r(:,3),'r--', 'LineWidth',2)
% subplot(2,5,8), plot(squeeze(-IDloads(:,(nTrials_l+1):(nTrials_l+17),3)),'g')
% hold on
% subplot(2,5,8), plot(squeeze(-IDloads(:,(nTrials_l+18):end,3)),'b')
subplot(2,5,8), plot(-momentsMean_r(:,3),'r', 'LineWidth',2)
hold on
subplot(2,5,8), plot(-IDloadsMean_r(:,3),'k', 'LineWidth',2)
% % subplot(5,2,6);
% plot(-momentsMean_r(:,3),'r', 'LineWidth',2)
% hold on
% % subplot(5,2,6);
% plot(-IDloadsMean_r(:,3),'k', 'LineWidth',2)

title('Knee FE Moment Calibration Results')
xlabel('Time Frames')
ylabel('Moment (N-m)')
axis([0 100 -50 100])
% legend({'Average Model Moment' 'Average ID Moment'});% 'Model Min/Max Moment' 'ID Min/Max Moment'})me
set(gca, 'FontSize', 14)

% figure;
subplot(2,5,9), plot(-PassiveMomentsMean_r(:,4),'r--', 'LineWidth',2)
% subplot(2,5,9), plot(squeeze(-IDloads(:,(nTrials_l+1):(nTrials_l+17),4)),'g')
% hold on
% subplot(2,5,9), plot(squeeze(-IDloads(:,(nTrials_l+18):end,4)),'b')
subplot(2,5,9), plot(-momentsMean_r(:,4),'r', 'LineWidth',2)
hold on
subplot(2,5,9), plot(-IDloadsMean_r(:,4),'k', 'LineWidth',2)
% % subplot(5,2,8);
% plot(-momentsMean_r(:,4),'r', 'LineWidth',2)
% hold on
% % subplot(5,2,8);
% plot(-IDloadsMean_r(:,4),'k', 'LineWidth',2)

title('Ankle FE Moment Calibration Results')
xlabel('Time Frames')
ylabel('Moment (N-m)')
axis([0 100 -50 100])
% legend({'Average Model Moment' 'Average ID Moment'});% 'Model Min/Max Moment' 'ID Min/Max Moment'})me
set(gca, 'FontSize', 14)

% figure;
subplot(2,5,10), plot(-PassiveMomentsMean_r(:,5),'r--', 'LineWidth',2)
% subplot(2,5,10), plot(squeeze(-IDloads(:,(nTrials_l+1):(nTrials_l+17),5)),'g')
% hold on
% subplot(2,5,10), plot(squeeze(-IDloads(:,(nTrials_l+18):end,5)),'b')
subplot(2,5,10), plot(-momentsMean_r(:,5),'r', 'LineWidth',2)
hold on
subplot(2,5,10), plot(-IDloadsMean_r(:,5),'k', 'LineWidth',2)
% % subplot(5,2,10);
% plot(-momentsMean_r(:,5),'r', 'LineWidth',2)
% hold on
% % subplot(5,2,10);
% plot(-IDloadsMean_r(:,5),'k', 'LineWidth',2)

title('Subtalar IE Moment Calibration Results')
xlabel('Time Frames')
ylabel('Moment (N-m)')
axis([0 100 -50 100])
% legend({'Average Model Moment' 'Average ID Moment'});% 'Model Min/Max Moment' 'ID Min/Max Moment'})
set(gca, 'FontSize', 14)
saveas(fig, 'MomentMatchingAverageCurve.jpg')

end

function makeMuscleParameterPlots(Lmt, lmtilda, vmtilda, HipFEMA, HipAAMA, KneeMA, AnkleMA, SubtalarMA,Lmtorig,HipMAorig,HipAAMAorig,KneeMAorig,AnkleMAorig,SubtalarMAorig,optParams)

nMuscTotal = optParams.nMuscTotal;
OpensimMuscleLabels = optParams.MuscLabels;

Lmtmean = permute(mean(Lmt,2),[1 3 2]);
lmtildamean = permute(mean(lmtilda,2),[1 3 2]);
vmtildamean = permute(mean(vmtilda,2),[1 3 2]);
HipMAmean = permute(mean(HipFEMA,2),[1 3 2]);
HipAAMAmean = permute(mean(HipAAMA,2),[1 3 2]);
KneeMAmean = permute(mean(KneeMA,2),[1 3 2]);
AnkleMAmean = permute(mean(AnkleMA,2),[1 3 2]);
SubtalarMAmean = permute(mean(SubtalarMA,2),[1 3 2]);

Lmtorigmean = permute(mean(Lmtorig,3),[1 2 3]);
HipMAorigmean = permute(mean(HipMAorig,3),[1 2 3]);
HipAAMAorigmean = permute(mean(HipAAMAorig,3),[1 2 3]);
KneeMAorigmean = permute(mean(KneeMAorig,3),[1 2 3]);
AnkleMAorigmean = permute(mean(AnkleMAorig,3),[1 2 3]);
SubtalarMAorigmean = permute(mean(SubtalarMAorig,3),[1 2 3]);

Lmtmax = permute(max(Lmt,[],2),[1 2 3]);
lmtildamax = permute(max(lmtilda,[],2),[1 3 2]);
vmtildamax = permute(max(vmtilda,[],2),[1 3 2]);
HipMAmax = permute(max(HipFEMA,[],2),[1 3 2]);
HipAAMAmax = permute(max(HipAAMA,[],2),[1 3 2]);
KneeMAmax = permute(max(KneeMA,[],2),[1 3 2]);
AnkleMAmax = permute(max(AnkleMA,[],2),[1 3 2]);
SubtalarMAmax = permute(max(SubtalarMA,[],2),[1 3 2]);

Lmtmin = permute(min(Lmt,[],2),[1 3 2]);
lmtildamin = permute(min(lmtilda,[],2),[1 3 2]);
vmtildamin = permute(min(vmtilda,[],2),[1 3 2]);
HipMAmin = permute(min(HipFEMA,[],2),[1 3 2]);
HipAAMAmin = permute(min(HipAAMA,[],2),[1 3 2]);
KneeMAmin = permute(min(KneeMA,[],2),[1 3 2]);
AnkleMAmin = permute(min(AnkleMA,[],2),[1 3 2]);
SubtalarMAmin = permute(min(SubtalarMA,[],2),[1 3 2]);

fig = figure('units','normalized','outerposition',[0 0 1 1]);
for i = 1:nMuscTotal
    subplot(6,6,i), plot(0:optParams.SampleStep:100,Lmtmean(:,i))
    hold on
    subplot(6,6,i), plot(0:optParams.SampleStep:100,Lmtmax(:,i),'r-.')
    subplot(6,6,i), plot(0:optParams.SampleStep:100,Lmtmin(:,i),'r-.')
    subplot(6,6,i), plot(0:optParams.SampleStep:100,Lmtorigmean(:,i),'g')
    title([OpensimMuscleLabels{i} ' mean Lmt'])
    %         axis([0 100 .4 1.4]);
    set(gca, 'FontSize', 10)
end

fig = figure('units','normalized','outerposition',[0 0 1 1]);
for i = 1:nMuscTotal
    subplot(6,6,i), plot(0:optParams.SampleStep:100,lmtildamean(:,i))
    hold on
    subplot(6,6,i), plot(0:optParams.SampleStep:100,lmtildamax(:,i),'r-.')
    subplot(6,6,i), plot(0:optParams.SampleStep:100,lmtildamin(:,i),'r-.')
    title([OpensimMuscleLabels{i} ' mean lmtilda'])
    axis([0 100 .4 1.4]);
    set(gca, 'FontSize', 10)
end
%     saveas(fig, 'Musclmtilda.jpg')

fig = figure('units','normalized','outerposition',[0 0 1 1]);
for i = 1:nMuscTotal
    subplot(6,6,i), plot(0:optParams.SampleStep:100,vmtildamean(:,i))
    hold on
    subplot(6,6,i), plot(0:optParams.SampleStep:100,vmtildamax(:,i),'r-.')
    subplot(6,6,i), plot(0:optParams.SampleStep:100,vmtildamin(:,i),'r-.')
    title([OpensimMuscleLabels{i} ' mean vmtilda'])
    axis([0 100 -1 1]);
    set(gca, 'FontSize', 10)
end
%     saveas(fig, 'Muscvmtilda.jpg')

fig = figure('units','normalized','outerposition',[0 0 1 1]);
for i = 1:nMuscTotal
    subplot(6,6,i), plot(0:optParams.SampleStep:100,HipMAmean(:,i))
    hold on
    subplot(6,6,i), plot(0:optParams.SampleStep:100,HipMAorigmean(:,i),'g')
    subplot(6,6,i), plot(0:optParams.SampleStep:100,HipMAmax(:,i),'r-.')
    subplot(6,6,i), plot(0:optParams.SampleStep:100,HipMAmin(:,i),'r-.')
    title([OpensimMuscleLabels{i} ' mean Hip FE MA'])
    set(gca, 'FontSize', 10)
end
%     saveas(fig, 'HipFEMA.jpg')

fig = figure('units','normalized','outerposition',[0 0 1 1]);
for i = 1:nMuscTotal
    subplot(6,6,i), plot(0:optParams.SampleStep:100,HipAAMAmean(:,i))
    hold on
    subplot(6,6,i), plot(0:optParams.SampleStep:100,HipAAMAorigmean(:,i),'g')
    subplot(6,6,i), plot(0:optParams.SampleStep:100,HipAAMAmax(:,i),'r-.')
    subplot(6,6,i), plot(0:optParams.SampleStep:100,HipAAMAmin(:,i),'r-.')
    title([OpensimMuscleLabels{i} ' mean Hip AA MA'])
    set(gca, 'FontSize', 10)
end
%     saveas(fig, 'HipAAMA.jpg')

fig = figure('units','normalized','outerposition',[0 0 1 1]);
for i = 1:nMuscTotal
    subplot(6,6,i), plot(0:optParams.SampleStep:100,KneeMAmean(:,i))
    hold on
    subplot(6,6,i), plot(0:optParams.SampleStep:100,KneeMAorigmean(:,i),'g')
    subplot(6,6,i), plot(0:optParams.SampleStep:100,KneeMAmax(:,i),'r-.')
    subplot(6,6,i), plot(0:optParams.SampleStep:100,KneeMAmin(:,i),'r-.')
    title([OpensimMuscleLabels{i} ' mean Knee MA'])
    set(gca, 'FontSize', 10)
end
%     saveas(fig, 'KneeMA.jpg')

fig = figure('units','normalized','outerposition',[0 0 1 1]);
for i = 1:nMuscTotal
    subplot(6,6,i), plot(0:optParams.SampleStep:100,AnkleMAmean(:,i))
    hold on
    subplot(6,6,i), plot(0:optParams.SampleStep:100,AnkleMAorigmean(:,i),'g')
    subplot(6,6,i), plot(0:optParams.SampleStep:100,AnkleMAmax(:,i),'r-.')
    subplot(6,6,i), plot(0:optParams.SampleStep:100,AnkleMAmin(:,i),'r-.')
    title([OpensimMuscleLabels{i} ' mean Ankle MA'])
    set(gca, 'FontSize', 10)
end
%     saveas(fig, 'AnkleMA.jpg')

fig = figure('units','normalized','outerposition',[0 0 1 1]);
for i = 1:nMuscTotal
    subplot(6,6,i), plot(0:optParams.SampleStep:100,SubtalarMAmean(:,i))
    hold on
    subplot(6,6,i), plot(0:optParams.SampleStep:100,SubtalarMAorigmean(:,i),'g')
    subplot(6,6,i), plot(0:optParams.SampleStep:100,SubtalarMAmax(:,i),'r-.')
    subplot(6,6,i), plot(0:optParams.SampleStep:100,SubtalarMAmin(:,i),'r-.')
    title([OpensimMuscleLabels{i} ' mean Subtalar MA'])
    set(gca, 'FontSize', 10)
end
%     saveas(fig, 'SubtalarMA.jpg')
end

function makeExcitationPlots(Excitations,a,optParams)

nTrials_l = optParams.nTrials_l;
nTrials_r = optParams.nTrials_r;
MuscleLabels = optParams.MuscLabels;

a = reshape(a,optParams.nptsShort,optParams.nTrials,optParams.nMuscTotal);

figure
ExcitationMeans = permute(mean(Excitations(:,1:nTrials_l,:),2),[1 3 2]);
aMeans = permute(mean(a(:,1:nTrials_l,:),2),[1 3 2]);
for iii = 1:optParams.nMuscEmg
    subplot(6,6,iii), plot(0:optParams.SampleStep:100,ExcitationMeans(:,iii),'LineWidth',2);
end
for iii = 1:optParams.nMuscTotal
    subplot(6,6,iii);
    hold on
    plot(0:optParams.SampleStep:100,aMeans(:,iii),'r','LineWidth',2);
    axis([0 100 0 1])
    title(['Left ' MuscleLabels{iii}]);
end

figure
ExcitationMeans = permute(mean(Excitations(:,(nTrials_l+1):(nTrials_l+nTrials_r),:),2),[1 3 2]);
aMeans = permute(mean(a(:,(nTrials_l+1):(nTrials_l+nTrials_r),:),2),[1 3 2]);
for iii = 1:optParams.nMuscEmg
    subplot(6,6,iii), plot(0:optParams.SampleStep:100,ExcitationMeans(:,iii),'LineWidth',2);
    
end
for iii = 1:optParams.nMuscTotal
    subplot(6,6,iii);
    hold on
    plot(0:optParams.SampleStep:100,aMeans(:,iii),'r','LineWidth',2);
    axis([0 100 0 1])
    title(['Right ' MuscleLabels{iii}]);
end

end

function makePassiveMomentMatchingPlots(optParams)
load DigitizedPassiveMoments.mat

% calculate passive moments to match with thelen curves (activation is
% zero)
optParams.a = optParams.aThelen;
optParams.Mat = optParams.MatThelen;
optParams.onesCol = optParams.onesColThelen;
optParams.nframesAll = optParams.nframesAllThelen;

[moments] = calcMoments(optParams,1);
JAnglesGait = optParams.JAnglesReshaped;


PassiveM{1} = reshape(PassiveM{1},numel(PassiveM{1})/4,4);
PassiveM{1}(:,4) = [];
% PassiveM{1}(45:end,:) = [];
PassiveM{1} = PassiveM{1}(:);

PassiveM{2} = reshape(PassiveM{2},numel(PassiveM{2})/4,4);
PassiveM{2}(92:121,:) = [];
PassiveM{2} = PassiveM{2}(:);

optParams.MomentsPassive = [PassiveM{1}; PassiveM{2}; PassiveM{3}];

JAngles{1} = reshape(JAngles{1},numel(JAngles{1})/(4*6),4,6);
JAngles{1}(:,4,:) = [];
% JAngles{1}(45:end,:) = [];
JAngles{1} = reshape(JAngles{1},numel(JAngles{1})/(6),6);

JAngles{2} = reshape(JAngles{2},numel(JAngles{2})/(4*6),4,6);
JAngles{2}(92:121,:,:) = [];
JAngles{2} = reshape(JAngles{2},numel(JAngles{2})/(6),6);

nHip = length(JAngles{1});
nKnee = length(JAngles{2});
nAnkle = length(JAngles{3});

optParams.nHip = nHip;
optParams.nKnee = nKnee;
optParams.nAnkle = nAnkle;

JAngles = [JAngles{1}; JAngles{2}; JAngles{3}]*pi/180;

JAngles(:,1) = JAngles(:,1)-0*pi/180;
JAngles(:,2) = JAngles(:,2)-0*pi/180;
JAngles(:,4) = JAngles(:,4)-0*pi/180;

JAngles = [JAngles; JAnglesGait];

optParams.JAngles = JAngles;

JAnglesH = JAngles(1:nHip,1)*180/pi;
momentsH = moments(1:nHip,1);
PassiveMH = PassiveM{1};

JAnglesH = reshape(JAnglesH,nHip/3,3);
momentsH = reshape(momentsH,nHip/3,3);
PassiveMH = reshape(PassiveMH,nHip/3,3);

JAnglesK = JAngles(nHip+1:nHip+nKnee,3)*180/pi;
momentsK = moments(nHip+1:nHip+nKnee,3);
PassiveMK = PassiveM{2};

JAnglesK = reshape(JAnglesK,nKnee/4,4);
momentsK = reshape(momentsK,nKnee/4,4);
PassiveMK = reshape(PassiveMK,nKnee/4,4);

JAnglesA = JAngles(nHip+nKnee+1:nHip+nKnee+nAnkle,4)*180/pi;
momentsA = moments(nHip+nKnee+1:nHip+nKnee+nAnkle,4);
PassiveMA = PassiveM{3};

JAnglesA = reshape(JAnglesA,nAnkle/4,4);
momentsA = reshape(momentsA,nAnkle/4,4);
PassiveMA = reshape(PassiveMA,nAnkle/4,4);



figure

subplot(3,1,1), plot(JAnglesH,momentsH,'LineWidth',2);
hold on
subplot(3,1,1), plot(JAnglesH,PassiveMH,'--','LineWidth',2);
axis([-15 80 -60 100])
legend('K=15', 'K=60', 'K=90', 'K=110')
title('Hip Passive Moment Matching')

subplot(3,1,2), plot(JAnglesK, momentsK,'LineWidth',2);
hold on
subplot(3,1,2), plot(JAnglesK,PassiveMK,'--','LineWidth',2);
axis([0 120 -30 30])
legend('H=0 A=20', 'H=0 A=-15', 'H=15 A=0', 'H=-15 A=20')
title('Knee Passive Moment Matching')

subplot(3,1,3),plot(JAnglesA, momentsA,'LineWidth',2);
hold on
subplot(3,1,3), plot(JAnglesA,PassiveMA,'--','LineWidth',2);
axis([-30 32 -50 0])
legend('K=0', 'K=15', 'K=60', 'K=110')
title('Ankle Passive Moment Matching')


end

function makeSynergyCommandPlots(synergyCommands_l,synergyCommands_r,optParams)
nptsShort = optParams.nptsShort;
nTrials_l = optParams.nTrials_l;
nTrials_r = optParams.nTrials_r;
nSyn = optParams.nSyn;

figure;
synergyCommands_l = reshape(synergyCommands_l,[nptsShort,nTrials_l,nSyn]);
for iii = 1:nSyn
    subplot(2,3,iii)
    plot(1:nptsShort,synergyCommands_l(:,:,iii));
end

figure;
synergyCommands_r = reshape(synergyCommands_r,[nptsShort,nTrials_r,nSyn]);
for iii = 1:nSyn
    subplot(2,3,iii)
    plot(1:nptsShort,synergyCommands_r(:,:,iii));
end

end

function makeJointAnglePlots(optParams)
nTrials_l = optParams.nTrials_l;
nTrials_r = optParams.nTrials_r;
jointAngles_l = optParams.jointAngles(:,:,1:nTrials_l)*180/pi;
jointAngles_r = optParams.jointAngles(:,:,(nTrials_l+1):(nTrials_l+nTrials_r))*180/pi;

figure;
subplot(2,5,1);
plot(1:101,squeeze(jointAngles_l(:,1,:)));
title('Left Hip FE Angle');
axis([0,101,-20,80])

subplot(2,5,2);
plot(1:101,squeeze(jointAngles_l(:,2,:)));
title('Left Hip AA Angle');
axis([0,101,-20,80])

subplot(2,5,3);
plot(1:101,squeeze(jointAngles_l(:,3,:)));
title('Left Knee FE Angle');
axis([0,101,-20,80])

subplot(2,5,4);
plot(1:101,squeeze(jointAngles_l(:,4,:)));
title('Left Ankle FE Angle');
axis([0,101,-20,80])

subplot(2,5,5);
plot(1:101,squeeze(jointAngles_l(:,5,:)));
title('Left Subtalar Angle');
axis([0,101,-20,80])

subplot(2,5,6);
plot(1:101,squeeze(jointAngles_r(:,1,:)));
title('Right Hip FE Angle');
axis([0,101,-20,80])

subplot(2,5,7);
plot(1:101,squeeze(jointAngles_r(:,2,:)));
title('Right Hip AA Angle');
axis([0,101,-20,80])

subplot(2,5,8);
plot(1:101,squeeze(jointAngles_r(:,3,:)));
title('Right Knee FE Angle');
axis([0,101,-20,80])

subplot(2,5,9);
plot(1:101,squeeze(jointAngles_r(:,4,:)));
title('Right Ankle FE Angle');
axis([0,101,-20,80])

subplot(2,5,10);
plot(1:101,squeeze(jointAngles_r(:,5,:)));
title('Right Subtalar Angle');
axis([0,101,-20,80])

end

function makeEmgPlots(emg,optParams)
nTrials_l = optParams.nTrials_l;
nMuscEmg = optParams.nMuscEmg;

figure;
for iii = 1:nMuscEmg
    subplot(6,6,iii)
    plot(1:101,squeeze(emg(21:121,iii,1:nTrials_l)));
end
figure;
for iii = 1:nMuscEmg
    subplot(6,6,iii)
    plot(1:101,squeeze(emg(21:121,iii,(nTrials_l+1):end)));
end

end

